Package: MCPAN
Type: Package
Title: Multiple Comparisons Using Normal Approximation
Version: 1.1-22
Date: 2026-05-05
Authors@R: c(person("Frank", "Schaarschmidt", role=c("aut","cre"), email="schaarschmidt@biostat.uni-hannover.de"),
 person("Daniel", "Gerhard", role="aut"),
 person("Martin", "Sill", role="aut"))
Author: Frank Schaarschmidt [aut, cre],
  Daniel Gerhard [aut],
  Martin Sill [aut]
Maintainer: Frank Schaarschmidt <schaarschmidt@biostat.uni-hannover.de>
Imports: mvtnorm, multcomp, graphics, stats, magic, MCMCpack, plyr
Description: Multiple contrast tests and simultaneous confidence
 intervals based on normal approximation. With implementations for
 binomial proportions in a 2xk setting (risk difference and odds ratio),
 poly-3-adjusted tumour rates, biodiversity indices (multinomial data) 
 and expected values under lognormal assumption. Approximative power 
 calculation for multiple contrast tests of binomial and Gaussian data.
License: GPL-2
NeedsCompilation: no
Packaged: 2026-05-07 07:28:44 UTC; root
Repository: CRAN
Date/Publication: 2026-05-08 05:20:35 UTC
Built: R 4.5.2; ; 2026-05-08 06:48:50 UTC; unix

exportPattern("binomest.", "binomORci.", "binomRDci.", "binomRDtest.", "binomRRci.")

export(binomest, binomORci, binomRDci, binomRDtest, binomRRci, multinomORci,
censsample,censsamplef,
corrMatgen,
lndest,lnrest,lndci, lnrci,PQln,
plotCI, plotCII,plotCI.default,plotCI.sci, plotCI.confint.glht, plotCI.sci.ratio,
poly3ci,poly3est, poly3estf, poly3table, poly3test,
powerbinom, powerbinomOR, powermcpn, powermcpt,
Shannon, estShannon, estShannonf, Shannonci,
Simpson, estSimpson, estSimpsonf, Simpsonci,
mosaicdiv,
SCSrank, Waldci, Waldtest

)

importFrom(mvtnorm, qmvt, pmvt, qmvnorm, pmvnorm, rmvnorm)
importFrom(multcomp, contrMat)
importFrom(MCMCpack, rdirichlet)
importFrom(magic, adiag)
importFrom(plyr, alply)

importFrom("graphics", "abline", "arrows", "axis", "box", "par", "plot.new", "strwidth")
importFrom("stats", "cov2cor", "glm", "median", "model.frame", "pnorm", "qnorm", "rchisq", "rnorm", "rweibull", "var", "binomial", "coefficients", "vcov")

S3method(plot, binomRDci)
S3method(plot, binomRRci)
S3method(plot, binomORci)
S3method(plot, poly3ci)
S3method(plot, sci)
S3method(plot, Shannonci)
S3method(plot, Simpsonci)

S3method(print, binomRDci)
S3method(print, binomRDtest)
S3method(print, binomRRci)
S3method(print, binomORci)
S3method(print, poly3ci)
S3method(print, poly3est)
S3method(print, poly3test)
S3method(print, Shannonci)
S3method(print, Simpsonci)
S3method(print, multinomORci)

S3method(summary, binomRDci)
S3method(summary, binomRDtest)
S3method(summary, binomRRci)
S3method(summary, binomORci)
S3method(summary, Shannonci)
S3method(summary, Simpsonci)

S3method(binomRDci, default)
S3method(binomRDci, formula)
S3method(binomRDci, matrix)
S3method(binomRDci, table)

S3method(binomORci, default)
S3method(binomORci, formula)
S3method(binomORci, matrix)
S3method(binomORci, table)

S3method(binomRDtest, default)
S3method(binomRDtest, formula)
S3method(binomRDtest, matrix)
S3method(binomRDtest, table)

S3method(binomRRci, default)
S3method(binomRRci, formula)
S3method(binomRRci, matrix)
S3method(binomRRci, table)

S3method(binomest, default)
S3method(binomest, formula)
S3method(binomest, table)

S3method(as.data.frame, multinomORci)

S3method(plotCI, sci.ratio)
S3method(plotCI, default)
S3method(plotCI, sci)
S3method(plotCI, confint.glht)

`print.binomRDtest` <-
function(x, digits=4, ...)
{

if(x$method=="Wald")
 {methvar<-"Wald"}
if(x$method=="ADD1")
 {methvar<-"Add-1"}
if(x$method=="ADD2")
 {methvar<-"Add-2"}

# The p-value of the maximum test

if(x$dist=="MVN")
{
cat("Union intersection test using",methvar," variance estimator:","\n")
cat("P-value of the maximum test:","\n")
print(round(x$pval, digits=digits))
cat(" ","\n")
}
else{
cat("Local p-values using",methvar," variance estimator:","\n")
cat(" ","\n")
}


# A table of testresults

testresult <- cbind(x$estimate, x$teststat, x$p.val.adj)

colnames(testresult) <- c("estimate","testat","p.val.adj") 

print(round(testresult, digits=digits))


cat(" ","\n")
cat("where proportions are the probability of the event", x$success, "\n")
cat(" ","\n")

invisible(x)

}

'binomRDtest' <- function(x,...){UseMethod("binomRDtest")}


'binomRDtest.default' <- function(x, n, names=NULL, type="Dunnett",
 cmat=NULL, method="Wald", alternative="two.sided", dist="MVN", ...)
{

aargs<-list(...)


# check x, n

    k <- length(x)
    if(k<2)
     {stop("x, n must contain at least two elements")}

    if(any(n<2)) 
     {stop("sample size must be at least 2 in each group")}   

    if(length(n)!=k)
     {stop("x, n must be of equal length")}

    if(!is.numeric(x) | !is.numeric(n))
     {stop("x, n must be numeric vectors")}


# include checks for validity of methods


# check the names
# if no names are given, available names of x are taken
# if names are specified, names of x/n are overwritten

    gnames<-names

    if(is.null(gnames))
     {
      if(is.null(names(x)))
       {gnames <- names(x) <- names(n) <- as.character(1:k)}
       else
        {gnames <- names(n) <- names(x)}
     }
     else
      {
       if(length(gnames)!=k)
        {stop("length of names does not fit to length of x,n")}
        else
         {names(x) <- names(n) <- as.character(gnames)}
      }

# check the contrast matrix

# check the contrast matrix

    if (is.null(cmat)) {
      if(type=="Dunnett") {
        if(is.null(aargs$base)){base<-1}
        else{base<-aargs$base}
        cmat <- contrMat(n=n, type=type, base=base)
       }
       else{cmat <- contrMat(n = n, type = type)}
    }
    else {
        if (!is.matrix(cmat) || ncol(cmat) != k)
         {stop("cmat must be a matrix with number of columns = number of groups")}
    }


    est <- binomest.default(x=x, n=n, names=gnames, method = method, success=aargs$success)

    out <- Waldtest(estp = est$estp,
     varp = est$varcor,
     cmat = cmat, 
     alternative = alternative,
     dist=dist)

    out$estimate <- cmat %*% est$estimate
    colnames(cmat) <- gnames
    out$x <- x
    out$n <- n
    out$p <- est$estimate
    out$success <- est$success
    out$names <- gnames
    out$method <- method
    out$cmat <- cmat
    class(out) <- "binomRDtest"
    return(out)
}


'binomRDtest.table' <- function(x, type="Dunnett",
 cmat=NULL, method="Wald", alternative="two.sided", dist="MVN", ...)
{

aargs<-list(...)


 est<-binomest.table(x, method=method, success=aargs$success)
 n<-est$n
 k<-length(n)

    if(any(n<2)) 
     {stop("Sample size must be at least 2 in each group")}   

    gnames<-est$names

# check the contrast matrix

 # check the contrast matrix

    if (is.null(cmat)) {
      if(type=="Dunnett") {
        if(is.null(aargs$base)){base<-1}
        else{base<-aargs$base}
        cmat <- contrMat(n=n, type=type, base=base)
       }
       else{cmat <- contrMat(n = n, type = type)}
    }
    else {
        if (!is.matrix(cmat) || ncol(cmat) != k)
         {stop("cmat must be a matrix with number of columns = number of groups")}
    }

    out <- Waldtest(estp = est$estp,
     varp = est$varcor,
     cmat = cmat, 
     alternative = alternative,
     dist=dist)

    out$estimate <- cmat %*% est$estimate
    colnames(cmat) <- gnames
    out$x <- est$Y
    out$n <- est$n
    out$p <- est$estimate
    out$success <- est$success
    out$names <- gnames
    out$method <- method
    out$cmat <- cmat

    class(out) <- "binomRDtest"
    return(out)
}




'binomRDtest.matrix' <- function(x, type="Dunnett",
 cmat=NULL, method="Wald", alternative="two.sided", dist="MVN",...)
{

 mat<-as.table(x)

 aargs<-list(...)

 aargs$x<-mat
 aargs$type<-type
 aargs$cmat<-cmat
 aargs$method<-method
 aargs$alternative<-alternative
 aargs$dist<-dist 

 out<-do.call("binomRDtest.table", args=aargs)
 
 return(out)
}




'binomRDtest.formula' <- function(formula, data, type="Dunnett",
 cmat=NULL, method="Wald", alternative="two.sided", dist="MVN", ...)
{

 testaargs<-aargs<-list(...)
 aargs$formula<-formula
 aargs$data<-data
 aargs$method<-method

 est<-do.call("binomest.formula", args=aargs)
 testaargs$x<-est$Y
 testaargs$n<-est$n
 testaargs$names<-est$names
 testaargs$type<-type
 testaargs$cmat<-cmat
 testaargs$method<-method
 testaargs$alternative<-alternative
 testaargs$dist<-dist 
 testaargs$success<-est$success

 out<-do.call("binomRDtest.default", args=testaargs)
 return(out)

}

"plotCII" <-
function(estimate, lower=NULL, upper=NULL, alternative=c("two.sided","less","greater"),
 lines=NULL, lineslty=2, lineslwd=1, linescol="black",
 CIvert=FALSE, CIlty = 1, CIlwd=1, CIcex=1, CIcol="black", CIlength=NULL,
 HL=TRUE, ...)
{
if(HL){
old.par <- par(no.readonly=TRUE)
}

aargs <- list(...)

# check input variables

if(length(estimate)<1 | (!is.numeric(estimate)&!is.integer(estimate)))
 {stop("Argument estimate should be a numeric vector")}

k<-length(estimate)
num <- 1:k

if(is.null(names(estimate)))
 {compn <- paste("C", num, sep="")}
else{compn <- names(estimate)}

if(!is.null(lower))
{
if(!is.numeric(lower)&!is.integer(lower))
 {stop("Argument lower should be a numeric vector")}
if(length(lower)!=k)
 {stop("Argument lower should be a vector of the same length as estimate!")}
}

if(!is.null(upper))
{
if(!is.numeric(upper)&!is.integer(upper))
 {stop("Argument upper should be a numeric vector")}
if(length(upper)!=k)
 {stop("Argument upper should be a vector of the same length as estimate!")}
}

alternative<-match.arg(alternative)

if(!is.null(lines))
{
if(!is.numeric(lines)&!is.integer(lines))
 {stop("Argument lines should be a numeric vector")}
}

CIlty<-rep(CIlty, length.out=k)
CIlwd<-rep(CIlwd, length.out=k)
CIcol<-rep(CIcol, length.out=k)

mymai <- par("mai")

# define the plot range

if(is.null(lower) & is.null(upper)){warning("No confidence limits specified: arguments lower and upper are both empty!")}

# define the plot range in 
# dependence of the alternative:

switch(alternative,
"two.sided"={
   allpoints <- c(lower, estimate, upper)
   if(all(!is.finite(allpoints))){stop("Arguments estimate, lower and upper contain only infinity or missing values!")}
   allexist<-allpoints[is.finite(allpoints)]
   lplot <- min(c(allexist, lines))
   uplot <- max(c(allexist, lines))
   },

"less"=
  {

  allpoints<-c(estimate, upper)
  if(all(!is.finite(allpoints))){stop("Arguments estimate and upper contain only infinity or missing values!")}
  allexist<-allpoints[is.finite(allpoints)]
  lplot <- min(c(lines, allexist)) 
  uplot <- max(c(allexist, lines))
  },

"greater"=
  {

  allpoints<-c(lower, estimate)
  if(all(!is.finite(allpoints))){stop("Arguments estimate and lower contain only infinity or missing values!")}
  allexist<-allpoints[is.finite(allpoints)]
  lplot <- min(c(lines, allexist)) 
  uplot <- max(c(lines, allexist))

  })

# Define the final plot ranges:

 llplot <- lplot - 0.1*abs(lplot-uplot)
 uuplot <- uplot + 0.1*abs(lplot-uplot)

# define the type of interval drawn,
# appropriate for unbounded CI

switch(alternative,

"two.sided"={

if(is.null(lower)){lower<-rep(llplot,k); code<-rep(2,k)
warning("No lower limits specified!")
}
else{
if(is.null(upper)){upper<-rep(uuplot,k); code<-rep(1,k)
warning("No upper limits specified!")
}
else{

code<-rep(3,k)

infl<-!is.finite(lower)
lower[infl]<-llplot
code[infl]<-2

infu<-!is.finite(upper)
upper[infu]<-uuplot
code[infu]<-1

infts <- infl&infu
code[infts]<-0
}}
},

"less"={
code<-rep(2,k)

if(is.null(upper)){upper<-rep(uuplot,k); code<-rep(0,k)
warning("No upper limits specified although alternative='less'!")
}

infu<-!is.finite(upper)
upper[infu]<-uuplot
code[infu]<-0

},

"greater"={
code<-rep(1,k)

if(is.null(lower)){lower<-rep(llplot,k); code<-rep(0,k)
warning("No lower limits specified although alternative='greater'!")
}

infl<-!is.finite(lower)
lower[infl]<-llplot
code[infl]<-0
})


# Define the defaults for main, sub, ylab, xlab:

if (is.null(aargs$main)) {aargs$main<-""} 
if (is.null(aargs$sub)) {aargs$sub<-""}
if (is.null(aargs$ylab)) {aargs$ylab<-""} 
if (is.null(aargs$xlab)) {aargs$xlab<-""}

# Box arguments

bargs<-list()
if(is.null(aargs$bty)){BTY<-"o"}
else{BTY<-aargs$bty}

# plot function for vertical CI:

if(CIvert==TRUE)
{

if(HL)
{
 plot.new()

 # adjust margin under the x axis according to length of comparison names

 xwidth<- 1.5 * max(strwidth(compn, units = "inches", cex = par("cex.axis"))) 

 if (mymai[1] < xwidth) 
        mymai[1] <- xwidth
 par(mai=mymai, new=TRUE)
}
aargs$x<-num
aargs$y<-estimate
aargs$axes<-FALSE

if(is.null(aargs$ylim)){aargs$ylim<-c(llplot, uuplot)}
if(is.null(aargs$type)){aargs$type<-"p"}
if(is.null(aargs$pch)){aargs$pch<-16}
if(is.null(aargs$cex)){aargs$cex<-CIcex}

do.call("plot", aargs)

axis(side = 1, at = num, labels=compn, las=2, ... )
axis(side=2, las=2, ...)
box(bty=BTY)

abline(v=num, col="lightgrey", lty=3)

if(!is.null(lines))
{
abline(h=lines, lty=lineslty, lwd=lineslwd, col=linescol)
}

if(is.null(CIlength))
{

arrlength<-1/(k*2)

#if(k<25)
# {arrlength<-0.08}
#else
# {arrlength<-0.05}
}
else{
arrlength<-CIlength
}

switch(alternative,

"two.sided"={

 for(i in 1:length(num))
  {
  arrows(x0=num[i], x1=num[i], y0=lower[i], y1=upper[i],
 length = arrlength, angle = 90, code = code[i],
       col = CIcol[i], lty = CIlty[i], lwd = CIlwd[i])

  }
 },

"less"={
 for(i in 1:length(num))
  {
  arrows(x0=num[i], x1=num[i], y0=llplot, y1=upper[i],
  length = arrlength, angle = 90, code = code[i],
       col = CIcol[i], lty = CIlty[i], lwd = CIlwd[i])
  }
 },

"greater"={

 for(i in 1:length(num))
  {
  arrows(x0=num[i], x1=num[i], y0=lower[i], y1=uuplot,
  length = arrlength, angle = 90, code = code[i],
       col = CIcol[i], lty = CIlty[i], lwd = CIlwd[i])
  }
 }
)


}


# plot function for horizontal CI:


if(CIvert==FALSE)
{
if(HL)
{
plot.new()

 # adjust margin under the x axis according to length of comparison names
  ywidth<- 1.5 * max(strwidth(compn, units = "inches", cex = par("cex.axis"))) 

 if (mymai[2] < ywidth) 
        mymai[2] <- ywidth
 par(mai=mymai, new=TRUE)
}

rnum<-rev(num)

aargs$y<-rnum
aargs$x<-estimate
aargs$axes<-FALSE

if(is.null(aargs$xlim)){aargs$xlim<-c(llplot, uuplot)}
if(is.null(aargs$type)){aargs$type<-"p"}
if(is.null(aargs$pch)){aargs$pch<-16}
if(is.null(aargs$cex)){aargs$cex<-CIcex}

do.call("plot", aargs)

axis(side = 2, at = rnum, labels=compn, las=2, ...)
axis(side = 1, ...)
box(bty=BTY)

abline(h=num, col="lightgrey", lty=3)

abline(v=lines, lty=lineslty, lwd=lineslwd, col=linescol)

if(is.null(CIlength)){arrlength<-1/(k*2)}
else{arrlength<-CIlength}

switch(alternative,

"two.sided"={
 for(i in 1:length(num))
  {

  arrows(y0=rnum[i], y1=rnum[i], x0=lower[i], x1=upper[i],
 length = arrlength, angle = 90, code = code[i],
       col = CIcol[i], lty = CIlty[i], lwd = CIlwd[i])

  }
 },

"less"={
 for(i in 1:length(num))
  {
  arrows(y0=rnum[i], y1=rnum[i], x0=llplot, x1=upper[i],
  length = arrlength, angle = 90, code = code[i],
       col = CIcol[i], lty = CIlty[i], lwd = CIlwd[i])
  }
 },

"greater"={
 for(i in 1:length(num))
  {
  arrows(y0=rnum[i], y1=rnum[i], x0=lower[i], x1=uuplot,
  length = arrlength, angle = 90, code = code[i],
       col = CIcol[i], lty = CIlty[i], lwd = CIlwd[i])
  }
 })


}

if(HL){
par(old.par)
}

}



#############################


plotCI<-function(x, ...){UseMethod("plotCI")}


#############################

plotCI.default<-function(x,...)
{

aargs<-list(...)

aargs$estimate<-x$estimate
aargs$lower<-x$conf.int[,1]
aargs$upper<-x$conf.int[,2]
aargs$alternative<-x$alternative

do.call("plotCII", aargs)

}

####


plotCI.sci<-function(x,...)
{
aargs<-list(...)
CI<-x$conf.int
cnames<-rownames(CI)

estimate<-as.numeric(x$estimate)
lower<-as.numeric(x$conf.int[,1])
upper<-as.numeric(x$conf.int[,2])

names(estimate)<-names(lower)<-names(upper)<-cnames

aargs$estimate<-estimate
aargs$lower<-lower
aargs$upper<-upper
aargs$alternative<-x$alternative

do.call(plotCII, args=aargs)

}





###########################################

plotCI.confint.glht<-function(x, ...)
{
aargs<-list(...)

CI<-x$confint
names<-rownames(CI)
estimate<-CI[,1]
names(estimate)<-names

aargs$estimate<-estimate
aargs$lower<-CI[,2]
aargs$upper<-CI[,3]
aargs$alternative<-x$alternative

do.call("plotCII", args=aargs)

}


# library(multcomp)

# amod <- aov(breaks ~ wool * tension, data = warpbreaks)
# wht <- glht(amod, linfct = mcp(tension = "Tukey"))
# CIwht<-confint(wht)
 
#windows()
#plotCI(CIwht, lines=c(-20,0,20), linescol=c("red","black","red"))

#windows()
#plot(CIwht)


#################################

plotCI.sci.ratio<-function(x, ...)
{
aargs<-list(...)

estimate<-as.numeric(x$estimate)
names(estimate)<-x$compnames
aargs$estimate<-estimate

CI<-x$conf.int

alternative<-x$alternative

switch(alternative,

"two.sided"={
aargs$lower<-CI[,1]
aargs$upper<-CI[,2]
},

"less"={
aargs$upper<-CI[,1]
},

"greater"={
aargs$lower<-CI[,1]
})

aargs$alternative<-alternative

do.call("plotCII", args=aargs)

}


#########################


plot.sci<-function(x,...)
{
aargs<-list(...)
CI<-x$conf.int
cnames<-rownames(CI)

estimate<-as.numeric(x$estimate)
lower<-as.numeric(x$conf.int[,1])
upper<-as.numeric(x$conf.int[,2])

names(estimate)<-names(lower)<-names(upper)<-cnames

aargs$estimate<-estimate
aargs$lower<-lower
aargs$upper<-upper
aargs$alternative<-x$alternative

do.call(plotCII, args=aargs)

}


plot.binomORci<-function(x,...)
{
aargs<-list(...)
CI<-x$conf.int
cnames<-rownames(CI)

estimate<-as.numeric(x$estimate)
lower<-as.numeric(x$conf.int[,1])
upper<-as.numeric(x$conf.int[,2])

names(estimate)<-names(lower)<-names(upper)<-cnames

aargs$estimate<-estimate
aargs$lower<-lower
aargs$upper<-upper
aargs$alternative<-x$alternative

do.call(plotCII, args=aargs)

}


plot.binomRRci<-function(x,...)
{
aargs<-list(...)
CI<-x$conf.int
cnames<-rownames(CI)

estimate<-as.numeric(x$estimate)
lower<-as.numeric(x$conf.int[,1])
upper<-as.numeric(x$conf.int[,2])

names(estimate)<-names(lower)<-names(upper)<-cnames

aargs$estimate<-estimate
aargs$lower<-lower
aargs$upper<-upper
aargs$alternative<-x$alternative

do.call(plotCII, args=aargs)

}



plot.binomRDci<-function(x,...)
{
aargs<-list(...)
CI<-x$conf.int
cnames<-rownames(CI)

estimate<-as.numeric(x$estimate)
lower<-as.numeric(x$conf.int[,1])
upper<-as.numeric(x$conf.int[,2])

names(estimate)<-names(lower)<-names(upper)<-cnames

aargs$estimate<-estimate
aargs$lower<-lower
aargs$upper<-upper
aargs$alternative<-x$alternative

do.call(plotCII, args=aargs)

}




plot.poly3ci<-function(x,...)
{
aargs<-list(...)
CI<-x$conf.int
cnames<-rownames(CI)

estimate<-as.numeric(x$estimate)
lower<-as.numeric(x$conf.int[,1])
upper<-as.numeric(x$conf.int[,2])

names(estimate)<-names(lower)<-names(upper)<-cnames

aargs$estimate<-estimate
aargs$lower<-lower
aargs$upper<-upper
aargs$alternative<-x$alternative

do.call(plotCII, args=aargs)

}




plot.Shannonci<-function(x,...)
{
aargs<-list(...)
CI<-x$conf.int
cnames<-rownames(CI)

estimate<-as.numeric(x$estimate)
lower<-as.numeric(x$conf.int[,1])
upper<-as.numeric(x$conf.int[,2])

names(estimate)<-names(lower)<-names(upper)<-cnames

aargs$estimate<-estimate
aargs$lower<-lower
aargs$upper<-upper
aargs$alternative<-x$alternative

do.call(plotCII, args=aargs)

}



plot.Simpsonci<-function(x,...)
{
aargs<-list(...)
CI<-x$conf.int
cnames<-rownames(CI)

estimate<-as.numeric(x$estimate)
lower<-as.numeric(x$conf.int[,1])
upper<-as.numeric(x$conf.int[,2])

names(estimate)<-names(lower)<-names(upper)<-cnames

aargs$estimate<-estimate
aargs$lower<-lower
aargs$upper<-upper
aargs$alternative<-x$alternative

do.call(plotCII, args=aargs)

}








#library(mratios)
#data(Penicillin)

#CGM<-sci.ratio(diameter~strain, data=Penicillin, type="GrandMean", alternative="greater")

#plotCI(CGM, lines=c(0.95,0.98,1,1/0.98, 1/0.95),
# lineslty=c(3,2,1,2,3), lineslwd=c(1,1,1,2,2), linescol="red",
#CIlwd=2)


#CGM<-sci.ratio(diameter~strain, data=Penicillin, type="GrandMean", alternative="less")

#pdf("CIplot1.pdf", width=8, height=5)
#plotCI(CGM, lines=c( 1/0.95),
# lineslty=2, lineslwd=2, linescol="red",
#CIlwd=2)
#dev.off()


#data(angina)

#CGM<-sci.ratio(response~dose, data=angina, type="Dunnett", alternative="two.sided")

#pdf("CIplot2.pdf", width=8, height=5)
#plotCI(CGM, lines=c(0.8, 1/0.8),
# lineslty=2, lineslwd=2, linescol="black",
#CIlwd=2,
#main="Ratio of means relative to the mean of the control group"
#)
#dev.off()

lndest <-
function(x, f)
{
    f <- as.factor(f)
    nx <- tapply(x, f, length)
    lx <- log(x)
    mlx <- tapply(lx,f,mean)
    varlx <- tapply(lx,f,var)*((nx-1)/nx)
    mx <- exp(mlx + 0.5 * varlx)
    varmx <- (mx^2)*varlx/nx + (mx^2)*(varlx^2)/(2*nx)
return(list(estimate=mx, varest=varmx, n=nx))}

'binomRDci' <- function(x,...){UseMethod("binomRDci")}

'binomRDci.default' <- function(x, n, names=NULL, type="Dunnett",
 cmat=NULL, method="Wald", alternative="two.sided", conf.level=0.95, dist="MVN", ...)
{

aargs<-list(...)

# check x, n

    k <- length(x)
    if(k<2)
     {stop("x, n must contain at least two elements")}

    if(any(n<2)) 
     {stop("sample size must be at least 2 in each group")}   

    if(length(n)!=k)
     {stop("x, n must be of equal length")}

    if(!is.numeric(x) | !is.numeric(n))
     {stop("x, n must be numeric vectors")}

# include checks for validity of methods



# check the names
# if no names are given, available names of x are taken
# if names are specified, names of x/n are overwritten

    gnames<-names

    if(is.null(gnames))
     {
      if(is.null(names(x)))
       {gnames <- names(x) <- names(n) <- as.character(1:k)}
       else
        {gnames <- names(n) <- names(x)}
     }
     else
      {
       if(length(gnames)!=k)
        {stop("length of names does not fit to length of x,n")}
        else
         {names(x) <- names(n) <- as.character(gnames)}
      }

# check the contrast matrix

    if (is.null(cmat)) {
      if(type=="Dunnett") {
        if(is.null(aargs$base)){base<-1}
        else{base<-aargs$base}
        cmat <- contrMat(n=n, type=type, base=base)
       }
       else{cmat <- contrMat(n = n, type = type)}
    }
    else {
        if (!is.matrix(cmat) || ncol(cmat) != k)
         {stop("cmat must be a matrix with number of columns = number of groups")}
    }

# get the point and variance estimates

    est <- binomest.default(x=x, n=n, names=gnames, method = method, success=aargs$success)

    out <- Waldci(estp = est$estp,
     varcor=est$varcor,
     varp = est$varp,
     cmat = cmat, 
     alternative = alternative,
     conf.level=conf.level,
     dist=dist)

    out$estimate <- cmat %*% est$estimate
    colnames(cmat) <- gnames
    out$x <- x
    out$n <- n
    out$p <- est$estimate
    out$success <- est$success
    out$names <- gnames
    out$method <- method
    out$cmat <- cmat


    class(out) <- c("binomRDci", "sci")
    return(out)
}




'binomRDci.table' <- function(x, type="Dunnett",
 cmat=NULL, method="Wald", alternative="two.sided", conf.level=0.95, dist="MVN",...)
{

 aargs<-list(...)

 est<-binomest.table(x, method=method, success=aargs$success)

 n<-est$n
 k<-length(n)

   if(any(n<2)) 
     {stop("Sample size must be at least 2 in each group")}   

    gnames<-est$names

# check the contrast matrix

    if (is.null(cmat)) {
      if(type=="Dunnett") {
        if(is.null(aargs$base)){base<-1}
        else{base<-aargs$base}
        cmat <- contrMat(n=n, type=type, base=base)
       }
       else{cmat <- contrMat(n = n, type = type)}
    }
    else {
        if (!is.matrix(cmat) || ncol(cmat) != k)
         {stop("cmat must be a matrix with number of columns = number of groups")}
    }

    out <- Waldci(estp = est$estp,
     varcor=est$varp,
     varp = est$varcor,
     cmat = cmat, 
     alternative = alternative,
     conf.level=conf.level,
     dist=dist)

    out$estimate <- cmat %*% est$estimate
    colnames(cmat) <- gnames
    out$x <- est$Y
    out$n <- est$n
    out$p <- est$estimate
    out$success <- est$success
    out$names <- gnames
    out$method <- method
    out$cmat <- cmat

    class(out) <- "binomRDci"
    return(out)
}


'binomRDci.matrix' <- function(x, type="Dunnett",
 cmat=NULL, method="Wald", alternative="two.sided", conf.level=0.95, dist="MVN",...)
{

 mat<-as.table(x)

 out<-binomRDci.table(mat,
 type=type, cmat=cmat,
 method=method, alternative=alternative, conf.level=conf.level, dist=dist, ...)
    return(out)
}



'binomRDci.formula' <- function(formula, data, type="Dunnett",
 cmat=NULL, method="Wald", alternative="two.sided", conf.level=0.95, dist="MVN",...)
{

 aargs<-list(...)
 aargs$formula<-formula
 aargs$data<-data
 aargs$method<-method

 est<-do.call("binomest.formula", args=aargs)

 out<-binomRDci.default(x=est$Y, n=est$n, names=est$names,
 type=type, cmat=cmat,
 method=method, alternative=alternative, conf.level=conf.level, success=est$success, dist=dist)
    return(out)

}

`poly3estf` <-
function(time, status, tmax=NULL, f, method="BP", k=NULL)
{

if(all(c("BP","BW","ADD1","ADD2")!=method))
 {stop("argument method must be one of 'BP','BW','ADD1','ADD2'")}

Ntot <- length(time)
#faclev <- levels(f)

if(length(status)!=Ntot || length(f)!=Ntot)
 {stop("time, status, f must not differ in length")}

if(is.null(tmax))
 {tmax <- max(time)}

if(is.null(k))
 {k<-3}

if(length(k)!=1)
 {k<-k[1]
  warning("Note: Only the first value of k has been used in the calculations!")
 }


timelist <- split(x=time, f=f, drop=TRUE)
statuslist <- split(x=status, f=f, drop=TRUE)

group <- names(timelist)

Ind<-length(timelist)

 Y <- numeric(length=Ind)
 n <- numeric(length=Ind)
 estp <- numeric(length=Ind)
 nadj <- numeric(length=Ind)
 varp <- numeric(length=Ind)
 varcor <- numeric(length=Ind)
 estimate <- numeric(length=Ind)
for (i in 1:length(timelist))
 {
 temp <- poly3est(time=timelist[[i]], status=statuslist[[i]], tmax=tmax, method=method, k=k)

 Y[i] <- temp$Y
 n[i] <- temp$n
 estp[i] <- temp$estp
 nadj[i] <- temp$nadj
 varp[i] <- temp$varp
 varcor[i] <- temp$varcor
 estimate[i] <- temp$estimate
 }

# H0pool <- poly3est(time=time, status=status, tmax=tmax)

# additional information

# earliest death
    mintime<-unlist(lapply(timelist,min))
# median time of death 
    medtime<-unlist(lapply(timelist, median))
# proportion death before tmax
    ntimebe<-lapply(timelist,function(x){length(which(x<tmax))})
    proptimebe<-unlist(ntimebe)/n

names(Y) <- names(n) <- names(estp) <- names(varp) <- names(varcor) <- group

out<-list(
Y=Y,
n=n,
estimate=estimate,
estp=estp,
nadj=nadj,
varp=varp,
varcor=varcor,
k=k,
names=group, 
method=method,
mintime=mintime,
medtime=medtime,
proptimebe=proptimebe
)

class(out)<-"poly3est"
return(out)

}

# Summary function

summary.poly3est<-function(object, ...)
{

args<-list(...)

if(is.null(args$digits) || !is.integer(args$digits))
 {digits<-4}
 else
  {digits<-args$digits}

    if (object$method == "BP") {
        methvar <- "Bailer-Portier"
    }
    if (object$method == "BW") {
        methvar <- "Bieler-Williams"
    }
    if (object$method == "ADD1") {
        methvar <- "Add-1"
    }
    if (object$method == "ADD2") {
        methvar <- "Add-2"
    }


 sample.estimate <- rbind(object$Y, object$n,
  object$Y/object$n,
 round(object$nadj, digits=digits),
 round(object$estimate, digits=digits),
 object$mintime,
 object$medtime,
 round(object$proptimebe,digits=digits) )

    rownames(sample.estimate) <- c("Number of tumours", "Number of animals",
 "Tumour proportion", "Adjusted number of animals",
 "Adjusted proportion",
 "First death",
 "Median time of death",
 "Proportion dead before end")

    cat("Raw and poly-", object$k, "-adjusted sample estimates:", 
        "\n", sep = "")

    print(sample.estimate)
    cat(" ", "\n")
    invisible(object)

}

"summary.binomRDtest"<-function(object,...)
{

aargs<-list(...)
if(is.null(aargs$digits))
 {digits<-4}
else
 {digits<-aargs$digits}


cat("Summary statistics: \n")

summarystat<-rbind("number of successes"=object$x,
"number of trials"=object$n,
"estimated success probability"=round(object$x/object$n, digits=digits))

colnames(summarystat)<-object$names

print(summarystat, digits=digits)


cat("\n Contrast matrix: \n")

print(object$cmat, digits=digits)

cat("\n")

print(object, digits=digits)

invisible(object)

}

`binomRRci` <-
function(x, ...){UseMethod("binomRRci")}


# # # THis is the extension of Gart-Nams (1988) "Crude" interval to the situation
# of multiple contrasts

# With the following changes:

# 1)
# If all x going to the denominator AND numerator of the contrast are ==0
# --> [lower, upper]<-[0, Inf]

# 2)
# If all x going to the numerator of the contrast are == 0
# --> [lower,]<-0

# 3)
# If all x going to the denominator of the contrast are == 0
# --> [,upper]<-Inf





`binomRRci.default` <-
function(x, n, names=NULL, type="Dunnett",
 cmat=NULL, alternative="two.sided", conf.level=0.95, dist="MVN", ...)
{

aargs<-list(...)

# check x, n

    k <- length(x)

    if(k<2)
     {stop("x, n must contain at least two elements")}

    if(any(n<2)) 
     {stop("sample size must be at least 2 in each group")}   

    if(length(n)!=k)
     {stop("x, n must be of equal length")}

    if(!is.numeric(x) | !is.numeric(n))
     {stop("x, n must be numeric vectors")}


#### RESTRICT BOUNDS function

restrictbounds<-function(x, cmat, conf.int)
{

if(all(x!=0)) {return(conf.int)}
else{

M<-nrow(cmat)

warning("0 occured in the data and the risk ratio might not be defined")

for(i in 1:M)
{

# all 0
if( all( x[sign(cmat[i,])!=0]==0 ) )
 {conf.int[i,1]<-0; conf.int[i,2]<-Inf}

# all numerator 0

if(all ( x[sign(cmat[i,])==1]==0 ) )
 {conf.int[i,1]<-0}


# all denominator 0

if(all (x[sign(cmat[i,])==(-1)]==0 ) )
 {conf.int[i,2]<-Inf}

}

return(conf.int)
}

}




# check the names
# if no names are given, available names of x are taken
# if names are specified, names of x/n are overwritten

    gnames<-names

    if(is.null(gnames))
     {
      if(is.null(names(x)))
       {gnames <- names(x) <- names(n) <- as.character(1:k)}
       else
        {gnames <- names(n) <- names(x)}
     }
     else
      {
       if(length(gnames)!=k)
        {stop("length of names does not fit to length of x,n")}
        else
         {names(x) <- names(n) <- as.character(gnames)}
      }

# check the contrast matrix

    if (is.null(cmat)) {
      if(type=="Dunnett") {
        if(is.null(aargs$base)){base<-1}
        else{base<-aargs$base}
        cmat <- contrMat(n=n, type=type, base=base)
       }
       else{cmat <- contrMat(n = n, type = type)}
    }
    else {
        if (!is.matrix(cmat) || ncol(cmat) != k)
         {stop("cmat must be a matrix with number of columns = number of groups")}
    }


# get the point and variance estimates

    est <- binomest.default(x=x, n=n, names=gnames, success=aargs$success)

  #  print(est)

if(any(x<4) & any(n-x<4))
 {warning("Normal approximation might be inappropriate")}


    xI<-est$Y+0.5
    nI<-est$n+0.5

    estI<-log(xI/nI)
    varI<- (1/xI)-(1/nI)

    out<-Waldci( cmat=cmat,
     estp=estI,
     varp=varI,
     varcor=varI,
     alternative = alternative,
     conf.level=conf.level,
     dist=dist)


    conf.int <- exp(out$conf.int)
    estimate <- exp(cmat %*% estI)

    if(!is.null(dimnames(cmat)[[1]]))
    {
    cnamesD<-dimnames(cmat)[[1]]
    cnamesSlist<-strsplit(cnamesD, "-")
    cnamesRR<-unlist(lapply(cnamesSlist, FUN=function(x){paste(x, collapse="/")}))
    }
    else{cnamesRR<-paste("C",1:nrow(cmat),sep="")}

    rownames(estimate)<-cnamesRR
    rownames(conf.int)<-cnamesRR    

    # expand the CI to the estimate:
    conf.int<-restrictbounds(x=x, cmat=cmat, conf.int=conf.int)

    out$conf.int<-conf.int
    out$estimate<-estimate
    colnames(cmat) <- gnames
    out$x <- x
    out$n <- n
    out$p <- est$estimate
    out$success <- est$success
    out$names <- gnames
    out$method <- "glm"
    out$cmat <- cmat
    out$method <-"Crude normal approximation for the risk ratio"

    class(out) <- c("binomRRci", "sci")
    return(out)
}


`binomRRci.formula` <-
function(formula, data, type="Dunnett",
 cmat=NULL, alternative="two.sided", conf.level=0.95, dist="MVN", ...)
{
aargs<-list(...)

est<-binomest.formula(formula=formula, data=data, success=aargs$success)

aargs$x<-est$Y
aargs$n<-est$n
aargs$names<-est$names
aargs$type<-type
aargs$cmat<-cmat
aargs$alternative<-alternative
aargs$conf.level<-conf.level
aargs$success<-est$success
aargs$dist<-dist

out<-do.call("binomRRci.default", aargs)

return(out)

}




`binomRRci.table` <-
function(x, type="Dunnett",
 cmat=NULL, alternative="two.sided", conf.level=0.95, dist="MVN", ...)
{
aargs<-list(...)

est<-binomest.table(x=x, success=aargs$success)

aargs$x<-est$Y
aargs$n<-est$n
aargs$names<-est$names
aargs$type<-type
aargs$cmat<-cmat
aargs$alternative<-alternative
aargs$conf.level<-conf.level
aargs$success<-est$success
aargs$dist<-dist

out<-do.call("binomRRci.default", aargs)

return(out)

}

`binomRRci.matrix` <-
function(x, type="Dunnett",
 cmat=NULL, alternative="two.sided", conf.level=0.95, dist="MVN", ...)
{
aargs<-list(...)

tab<-as.table(x)

est<-binomest.table(x=tab, success=aargs$success)

aargs$x<-est$Y
aargs$n<-est$n
aargs$names<-est$names
aargs$type<-type
aargs$cmat<-cmat
aargs$alternative<-alternative
aargs$conf.level<-conf.level
aargs$success<-est$success
aargs$dist<-dist

out<-do.call("binomRRci.default", aargs)

return(out)

}

`Simpson` <-
function(p)
{
1-sum(p^2)
}

`estSimpson` <-
function(x)
{
n<-sum(x)
estp<-x/n
Simpson(estp)*n/(n-1)
}

`estSimpsonf` <-
function(X, f)
{

if(any(floor(X)!=X))
 {warning("The elements of X should be integers!")}

J<-length(f)

if(nrow(X)!=J)
 {stop("The number of columns in X must be equal to the length of f!")}

varestSimpson<-function(x)
{
n<-sum(x)
estp<-x/n
S2<-sum(estp^2)
S3<-sum(estp^3)

2*(S2 + 2*(n-2)*S3 + (3-2*n)*S2^2)/(n*(n-1))
}

X<-as.data.frame(X)

names<-colnames(X)

ff<-as.factor(f)
Xs<-split(X, f=ff)

Xcs<-lapply(X=Xs, FUN=function(x){apply(X=x, MARGIN=2, FUN=sum)})

tab<-matrix(ncol=ncol(X), nrow=length(Xcs))
for(i in seq(along.with=Xcs))
{
tab[i,]<-Xcs[[i]]
}

colnames(tab)<-names
rownames(tab)<-names(Xcs)

estlist<-lapply(X=Xcs, FUN=function(x){estSimpson(x)})
estv<-unlist(estlist)
 
varestlist<-lapply(X=Xcs, FUN=function(x){varestSimpson(x)})
varestv<-unlist(varestlist)

return(list(estimate=estv, varest=varestv, table=tab))

}

`Simpsonci` <-
function(X, f, cmat=NULL, type="Dunnett", alternative = "two.sided", conf.level = 0.95, dist = "MVN", ...)
{
aargs<-list(...)

type<-match.arg(type, choices=c("Dunnett","Tukey","Sequen"))
alternative<-match.arg(alternative, choices=c("two.sided","less","greater"))

if(length(conf.level)!=1)
 {stop("argument conf.level should be a single numeric value")}

if(conf.level<=0.5|conf.level>=1) 
 {stop("argument conf.level should be a single numeric value between 0.5 and 1")}

if(!is.numeric(conf.level)) 
 {stop("argument conf.level should be a single numeric value")}

if(!is.data.frame(X))
 {stop("X must be of an object of class 'data.frame'")}

if(!is.factor(f))
 {f<-as.factor(f)}

k<-length(levels(f))

if(k<=1)
 {stop("The factor variable f should have at least 2 levels to be compared")}

esti<-estSimpsonf(X=X, f=as.factor(f))

n<-apply(X=esti$table, MARGIN=1, FUN=sum)

if(is.null(cmat))
 {
  if(is.null(aargs$base)){base<-1}
   else{base<-aargs$base}
 cmat<-contrMat(n=n, type=type, base=base)}
else
 {
 if(ncol(cmat)!=k)
  {stop("Number of columns in cmat should be the same as the number of levels in f")}
 }

out <- Waldci(cmat=cmat, estp=esti$estimate, varp=esti$varest,
 varcor=esti$varest, alternative = alternative, 
    conf.level = conf.level, dist = dist) 

TAB<-esti$table

estimate <- cmat %*% matrix(esti$estimate,ncol=1)

out$estimate <- estimate
out$cmat <-cmat
out$methodname <- "Confidence intervals for differences of Simpsons Indices"
out$sample.estimates <- esti


class(out)<-c("Simpsonci", "sci")
return(out)

}



`print.Simpsonci` <-
function(x,...)
{

# A table of confidence intervals

aargs<-list(...)

if(is.null(aargs$digits))
 {digits<-4}
else
 {digits<-aargs$digits}

dist<-attr(x$quantile, which="dist")

if(dist=="MVN"){
cat("Simultaneous", round(x$conf.level*100,3),"percent-confidence intervals","\n",
"for differences of Simpson indices", "\n")
}
else{
cat("Local", round(x$conf.level*100,3),"percent-confidence intervals","\n",
"for differences of Simpson indices", "\n")
}

conf.int <- cbind(x$estimate, x$conf.int)

colnames(conf.int)[1] <- "estimate"

print(round(conf.int, digits=digits))

invisible(x)
}


`summary.Simpsonci` <-
function(object,...)
{

aargs<-list(...)
if(is.null(aargs$digits))
 {digits<-4}
else
 {digits<-aargs$digits}

cat("\n Data: \n")

print(object$sample.estimates$table)

cat("\n Summary statistics: \n")

N<-apply(X=object$sample.estimates$table, MARGIN=1, FUN=function(x){sum(x)})

summarystat<-rbind(
"Total number of individuals"=round(N,0),
"Estimated Simpson index"=round(object$sample.estimates$estimate, digits),
"Variance estimate"=round(object$sample.estimates$varest,digits)
)

colnames(summarystat)<-names(object$sample.estimate$estimate)

print(summarystat, digits=digits)

cat("\n Contrast matrix: \n")

print(object$cmat, digits=digits)

cat("\n")

print(object, digits=digits)

invisible(object)
}

SCSrank <-
function(x, conf.level=0.95, alternative="two.sided", ...)
{
alternative <- match.arg(alternative, choices=c("two.sided","less","greater"))

DataMatrix <- x
N <- nrow(DataMatrix)
k <- round(conf.level*N,0)
RankDat <- apply(DataMatrix,2,rank)

switch(alternative,

"two.sided"={
W1 <- apply(RankDat,1,max)
W2 <- N + 1 - apply(RankDat,1,min)

Wmat <- cbind(W1,W2)
w <- apply(Wmat,1,max)
tstar <- round(sort(w)[k],0)

SCI <- function(x)
{
 sortx <- sort(x)
 cbind(sortx[N+1-tstar],sortx[tstar])
}

SCS <- t(apply(DataMatrix,2,SCI))
},

"less"={
W1 <- apply(RankDat,1,max)
tstar <- round(sort(W1)[k],0)

SCI <- function(x)
{
 sortx <- sort(x)
 cbind(-Inf, sortx[tstar])
}

SCS<-t(apply(DataMatrix,2,SCI))
},

"greater"={
W2 <- N + 1 - apply(RankDat,1,min)
tstar <- round(sort(W2)[k],0)

SCI <- function(x)
{
 sortx <- sort(x)
 cbind(sortx[N+1-tstar], Inf)
}

SCS<-t(apply(DataMatrix,2,SCI))

}
)
# end of switch

colnames(SCS)<-c("lower","upper")

attr(SCS, which="k")<-k
attr(SCS, which="N")<-N
OUT<-list(conf.int=SCS, conf.level=conf.level, alternative=alternative)
return(OUT)
}

#

mosaicdiv<-function(x, decreasing=NULL, ...)
{
if(!any(class(x)%in%c("Shannonci","Simpsonci"))){warning("This is a convenience function for objects of class 'Shannonci' or 'Simpsonci'!")}

tab<-x$sample.estimate$table

aargs<-list(...)

if(is.null(aargs$las))
 {aargs$las<-1}

if(is.null(aargs$main))
 {aargs$main<-""}


if(is.null(decreasing))
{
aargs$x<-tab
do.call("mosaicplot", aargs)
}
else{

if(!is.logical(decreasing)) 
 {stop("decreasing should be either TRUE, FALSE or NULL")}

if(decreasing)
{
cs<-apply(tab, 2, sum)
CO<-order(cs, decreasing=TRUE)
aargs$x<-tab[,CO]
do.call("mosaicplot", aargs)
}


if(!decreasing)
{
cs<-apply(tab, 2, sum)
CO<-order(cs, decreasing=FALSE)
aargs$x<-tab[,CO]
do.call("mosaicplot", aargs)
}
}

}

`corrMatgen` <-
function(CM,varp)

      {
      k <- ncol(CM)
      if( length(varp) != k )
       {stop("number of columns and length of var must be the same")}


    l <- nrow(CM)

      cor <- matrix(0, ncol=l, nrow=l)

    for(i in 1:l)
    {
      for(j in 1:l)
      {
        cor[i,j]<-sum( CM[i,]*CM[j,]*varp ) / sqrt( sum( (CM[i,]^2)*varp ) * sum( (CM[j,]^2)*varp ) )
       }

    }

      return(cor)
    }

lndci <-
function(x, f, type="Dunnett", cmat=NULL, alternative=c("two.sided","less","greater"), conf.level=0.95, method=c("GPQ","AN"), ...)
{
# additional arguments
aargs<-list(...)

# check arguments
if(!is.numeric(conf.level) || (length(conf.level)!=1| conf.level>=1| conf.level<0.5)){stop("conf.level should be a single numeric value smaller than 1 and larger than 0.5")}

alternative <- match.arg(alternative)
method <- match.arg(method)

if(!is.null(aargs$B) & method=="AN"){warning("Additional argument B will be ignored with method='AN'.")}
if(!is.null(aargs$dist) & method=="GPQ"){warning("Additional argument dist will be ignored with method='GPQ'.")}
wc <- which(names(aargs) %in% c("dist","B"))
if(length(aargs)>length(wc)){
if(length(wc)==0){warning(paste("Argument(s)", paste(names(aargs), collapse=", ") , "will be ignored."))}
if(length(wc)>0){warning(paste("Argument(s)", paste(names(aargs[-wc]), collapse=", ") , "will be ignored."))}
}

if(!is.null(aargs$B) & method=="GPQ"){
if(!is.numeric(aargs$B) & !is.integer(aargs$B)){stop("B must be a single integer value.")}
if(length(aargs$B)!=1){stop("B must be a single integer value.")}
}

ff <- droplevels(as.factor(f))
ni <- tapply(ff,ff, length)
ngroups<-length(ni)

if(any(ni<=3)){warning("At least one group has only 3 or less observations! Methods may be unreliable.")}

if(any(x<=0)){warning("Values of the response variable are less or equal 0!")}

    if (is.null(cmat)) {
      if(type=="Dunnett") {
        if(is.null(aargs$base)){base<-1}
        else{base<-aargs$base}
        cmat <- contrMat(n=ni, type=type, base=base)
       }
       else{cmat <- contrMat(n = ni, type = type)}
    }
    else {
        if (!is.matrix(cmat) || ncol(cmat) != ngroups)
         {stop("cmat must be a matrix with number of columns = number of groups")}
    }

EST <- lndest(x=x, f=f)
LEST <-  cmat %*% EST$estimate

# calculate SCI acc. to diff methods

switch(method,
"AN"={
if(is.null(aargs$dist)){dist<-"MVN"}else{dist<-aargs$dist}
SCI <- Waldci(cmat=cmat, estp=EST$estimate, varp=EST$varest, varcor=EST$varest, alternative = alternative, conf.level = conf.level, dist = dist) 
},
"GPQ"={
if(is.null(aargs$B)){B<-10000}else{B<-aargs$B}

lx<-log(x)
ldatlist<-split(x=lx, f=ff)

ni <- unlist(lapply(ldatlist, length))
mli <- unlist(lapply(ldatlist, mean))
varli <- unlist(lapply(ldatlist, var))

mi <- exp(mli + 0.5 * varli)
mimat<-matrix(mi, nrow=length(mi))

estimate<-cmat%*%mimat

PARMAT<-cbind(ni, mli, varli)

PQMAT<-apply(X=PARMAT, MARGIN=1, FUN=function(x){
PQln(nx=x[1], mlx=x[2], varlx=x[3], B=B)})

ePQMAT<-exp(PQMAT)

PQdiffs<-apply(X=ePQMAT, MARGIN=1, FUN=function(x){cmat %*% x})

SCI<-SCSrank(x=t(PQdiffs), alternative=alternative, conf.level=conf.level)

})

OUT<-c(list(estimate=LEST), SCI)

return(OUT)
}

`censsamplef` <-
function(n, scale.m, shape.m, scale.t, shape.t=3, tmax )

{
k <- length(n)

if( any(c(length(scale.m), length(shape.m), length(scale.t), length(shape.t))!=k))
 {stop("n, scale.m, shape.m, scale.t, shape.t must be vectors of equal length")}

names <- as.factor(rep( 1:k , times=c(n)))


dat<-data.frame()

for(i in 1:k)
{
 dat <- rbind(dat, 
 censsample(n=n[i], scale.m=scale.m[i], shape.m=shape.m[i], scale.t=scale.t[i], shape.t=shape.t[i], tmax=tmax ) )
}
dat$f <- names

return(dat)

}

`Waldci` <-
function(cmat, estp, varp, varcor, alternative="two.sided", conf.level=0.95, dist="MVN")

  {

k=ncol(cmat)
m=nrow(cmat)

if(any( c(length(estp), length(varp), length(varcor) )!=k ))
 { stop("lengths of estp, nadj, varp, varcor, and ncol(cmat) must be the same")}

    estC <- cmat %*% estp

    CorrMat<-corrMatgen(CM=cmat, varp=varcor)

varC <- (cmat^2) %*% (varp) 
  
dist<-match.arg(dist, choices=c("MVN", "N"))

switch(dist,

"MVN"={
 
    # Berechnung des MVNORM quantils

if(alternative=="two.sided")
 { quanti <- qmvnorm(p=conf.level, sigma=CorrMat, tail="both.tails")$quantile
   stderr <- sqrt(varC)
   lCI <- estC-quanti*stderr
   uCI <- estC+quanti*stderr
 }

else
 {
  if(alternative=="less")
   { quanti <- qmvnorm(p=conf.level, sigma=CorrMat, tail="lower.tail")$quantile
     stderr <- sqrt(varC)
     lCI <- rep(-Inf, m)
     uCI <- estC+quanti*stderr
   }

  else
   {
    if(alternative=="greater")
     {quanti <- qmvnorm(p=conf.level, sigma=CorrMat, tail="upper.tail")$quantile
      stderr <- sqrt(varC)
      lCI <- estC+quanti*stderr
      uCI <- rep(Inf, m)
     }
   }
 }
},

"N"={

if(alternative=="two.sided")
 { quanti <- qnorm(p=1-(1-conf.level)/2)
   stderr <- sqrt(varC)
   lCI <- estC-quanti*stderr
   uCI <- estC+quanti*stderr
 }

else
 {
  if(alternative=="less")
   { quanti <- qnorm(p=conf.level)
     stderr <- sqrt(varC)
     lCI <- rep(-Inf, m)
     uCI <- estC+quanti*stderr
   }

  else
   {
    if(alternative=="greater")
     {quanti <- qnorm(p=1-conf.level)
      stderr <- sqrt(varC)
      lCI <- estC+quanti*stderr
      uCI <- rep(Inf, m)
     }
   }
 }

})

conf.int<-cbind(lCI,uCI)
colnames(conf.int)<-c("lower","upper")

quantile<-quanti
attr(x=quantile, which="dist")<-dist

out<-list(conf.int=conf.int,
 alternative=alternative,
 conf.level=conf.level,
 quantile=quantile,
 corrmat=CorrMat,
 dist=dist)

class(out)<-"sci"

return(out)

  }

`print.poly3test` <-
function(x, digits=4, ...)
{


if(x$method=="BP")
 {methvar<-"Bailer-Portier"}
if(x$method=="BW")
 {methvar<-"Bieler-Williams"}
if(x$method=="ADD1")
 {methvar<-"Add-1"}
if(x$method=="ADD2")
 {methvar<-"Add-2"}


#  Table of sample estimates

sample.estimate<-rbind(x$sample.estimate$Y, x$sample.estimate$n, x$sample.estimate$nadj, x$sample.estimate$estimate)
rownames(sample.estimate)<-c("x","n", "adjusted n", "adjusted estimate")

cat("Sample estimates using poly-",x$k,"-adjustment","\n")
print(round(sample.estimate, digits=digits))
cat(" ","\n")

# The contrast matrix:

cat("Contrast matrix:","\n")
print(x$cmat)
cat(" ","\n")

if(x$dist=="MVN")
{
# The p-value of the maximum test
cat("Union-Intersection test using", methvar," variance estimator:","\n")
cat("P-value of the maximum test:","\n")
print(round(x$pval, digits=digits))
cat(" ","\n")
}
else{
cat("Local Wald test using",methvar," variance estimator:","\n")
}


# A table of testresults

testresult <- cbind(x$estimate, x$teststat, x$p.val.adj)

colnames(testresult) <- c("estimate","testat","p.val.adj") 

print(round(testresult, digits=digits))

cat(" ","\n")

invisible(x)

}

`summary.binomORci` <-
function(object,...)
{

aargs<-list(...)
if(is.null(aargs$digits))
 {digits<-4}
else
 {digits<-aargs$digits}

success<-object$success

cat("Summary statistics: \n")

summarystat<-rbind(object$x,
object$n,
round(object$x/object$n, digits=digits))

rownames(summarystat)<-c(paste("number of",success, collapse=" "),
"number of trials",
paste("estimated probability of",success, collapes=""))

colnames(summarystat)<-object$names

print(summarystat, digits=digits)


cat("\n Contrast matrix on the level of the logit link: \n")

print(object$cmat, digits=digits)

if(object$dist=="MVN")
{
cat("\n The estimated correlation matrix of the contrasts is: \n")

print(object$corrmat, digits=digits)

}

cat("\n")

print(object, digits=digits)

invisible(object)
}

`print.binomRRci` <-
function(x, digits=4, ...)
{

METHOD<-x$method
# A table of confidence intervals


dist<-attr(x$quantile, which="dist")

if(dist=="MVN"){
cat("Simultaneous", round(x$conf.level*100,3),"percent-confidence intervals","\n",
"for the ratio of proportions (RR), based on a crude normal approximation", "\n")
}
else{
cat("Local", round(x$conf.level*100,3),"percent-confidence intervals","\n",
"for the ratio of proportions (RR), based on a crude normal approximation", "\n")
}

conf.int <- cbind(x$estimate, x$conf.int)

colnames(conf.int)[1] <- "estimate"

print(round(conf.int, digits=digits))

cat(" ","\n")
cat("where proportions are the probabilities to observe", x$success, "\n")
cat(" ","\n")

invisible(x)
}

`poly3est` <-
function(time, status, tmax, method="BP", k=NULL)
{

if(is.null(k))
 {k<-3}

# default weight is one

weight <- rep(1,length(status))

    n<-length(status)
    Y<-sum(status)

# replace all weights where no tumor is present with the poly3weight (t.m/tmax)^3

for(i in which(!status))
 {weight[i] <- (time[i]/tmax)^k}

    nadj<-sum(weight)
    estimate<-Y/nadj 

if (method=="BP")
    {
    ##Varianzberechnung Bailer Portier
    estp<-estimate
    varp<-estp*(1-estp)/nadj 
    varcor<-varp

    if(Y==0)
    {
    estcor<-0.5/nadj
    varcor<-estcor*(1-estcor)/nadj
    }

    if(Y==n)
    {
    estcor<- (n-0.5)/nadj
    varcor<-estcor*(1-estcor)/nadj
    }
    
    

    }

if(method=="BW")
    {
    ##Varianzberechnung Bieler Williams 
    estp<-estimate
    resp <- as.numeric(status)  
    zet <- (resp - estp*weight)/nadj
    mzet <-mean(zet)
    varp <- (n/(n-1))*sum( (zet-mzet)^2 )
    #print("varp")
    #print(varp)
    varcor<-varp


    if(Y==0)
    {
    estcor <- 0.5/nadj 
#print(c(estcor,"estcor"))
    zet <- (resp - estcor*weight)/nadj
#print(c(zet,"zet"))
    mzet <-mean(zet)
#print(c(mzet,"mzet"))
    varcor <- (n/(n-1))*sum( (zet-mzet)^2 )
    #print("varcor1")
    #print(varcor)
    }


    if(Y==n)
    {
    estcor <- n-0.5/nadj 
    zet <- (resp - estcor*weight)/nadj
    mzet <-mean(zet)
    varcor <- (n/(n-1))*sum( (zet-mzet)^2 )
    }

}

if(method=="ADD1")
   {
   ##Varianzberechnung Bailer Portier mit add1
    estp <- (Y + 0.5)/(nadj+1)
    varp<-estp*(1-estp)/(nadj+1)
    varcor<-varp
   }


if(method=="ADD2")
   {
   ##Varianzberechnung Bailer Portier mit add1
    estp <- (Y + 1)/(nadj + 2)
    varp <- estp*(1-estp)/(nadj+2)
    varcor <- varp
   }

return(list(
 Y=Y,
 n=n,
 estimate=estimate,
 weight=weight,
 estp=estp,
 nadj=nadj,
 varp=varp,
 varcor=varcor,
 k=k))

}

powermcpn <-
function(ExpTeststat, corrH1, crit, alternative=c("two.sided", "less", "greater"), alpha=0.05, ptype=c("global", "anypair", "allpair"), ...)
{

M<-length(ExpTeststat)
CORR<-corrH1

switch(EXPR=ptype,
"global"={

switch(EXPR=alternative,
"two.sided"={beta <- pmvnorm(lower=rep(-crit,M), upper=rep(crit,M), mean=ExpTeststat, sigma=CORR,...); whichHA <- which(abs(ExpTeststat) > 10*.Machine$double.eps)},
"less"={beta <- pmvnorm(lower=rep(crit,M), upper=rep(Inf, M), mean=ExpTeststat, sigma=CORR,...); whichHA <- which(ExpTeststat < -10*.Machine$double.eps)},
"greater"={beta <- pmvnorm(lower=rep(-Inf,M), upper=rep(crit,M), mean=ExpTeststat, sigma=CORR,...); whichHA <- which(ExpTeststat > 10*.Machine$double.eps)})
},

"anypair"={

switch(EXPR=alternative,
"two.sided"={
whichHA <- which(abs(ExpTeststat) > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated."); beta <- 1-alpha}else{
beta <- pmvnorm(lower=rep(-crit, MHA), upper=rep(crit, MHA), mean=ExpTeststat[whichHA], sigma=CORR[whichHA,whichHA],...)}
},
"less"={
whichHA <- which(ExpTeststat < -10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated."); beta <- 1-alpha}else{
beta <- pmvnorm(lower=rep(crit, MHA), upper=rep(Inf, MHA), mean=ExpTeststat[whichHA], sigma=CORR[whichHA,whichHA],...)}
},
"greater"={
whichHA <- which(ExpTeststat > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated."); beta <- 1-alpha}else{
beta <- pmvnorm(lower=rep(-Inf,MHA), upper=rep(crit, MHA), mean=ExpTeststat[whichHA], sigma=CORR[whichHA,whichHA],...)}
})
},

"allpair"={

switch(EXPR=alternative,
"two.sided"={
whichHA <- which(abs(ExpTeststat) > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated."); beta<-1-alpha}else{
nsim <- 10000
RT <- rmvnorm(n=nsim, mean=ExpTeststat[whichHA], sigma=CORR[whichHA, whichHA], method="svd")
nreject <- sum( apply(RT, 1, function(x){min(abs(x))}) > abs(crit) )
beta <- 1 - (nreject/nsim)
simerror <- sqrt(beta*(1-beta)/nsim)
attr(beta, which="simerror")<-simerror
}},
"less"={
whichHA <- which(ExpTeststat < -10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated."); beta<-1-alpha}else{
beta <- 1 - pmvnorm(lower=rep(-Inf, MHA), upper=rep(crit,MHA), mean=ExpTeststat[whichHA], sigma=CORR[whichHA, whichHA],...)
}},
"greater"={
whichHA <- which(ExpTeststat > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated."); beta<-1-alpha}else{
beta <- 1 - pmvnorm(lower=rep(crit,MHA), upper=rep(Inf,MHA), mean=ExpTeststat[whichHA],  sigma=CORR[whichHA, whichHA],...)
}})
})


pow<- (1-beta)

HAtrue<-numeric(length=M)
HAtrue[whichHA]<-1

settings<-data.frame(ExpTstat=ExpTeststat, underHA=HAtrue)

out<-list(power=pow,
conexp=settings,
crit=crit,
alternative=alternative,
ptype=ptype,
alpha=alpha
)

return(out)
}

PQln <-
function(nx, mlx, varlx, B=10000)
{
    Zx<-rnorm(n=B, mean=0, sd=1)
    Chix<-rchisq(n=B, df=nx-1)
    Tx <- mlx - (Zx*sqrt(varlx))/((sqrt(Chix)/sqrt(nx-1))*sqrt(nx)) + (varlx)/(2*Chix/(nx-1))
    return(Tx)
}

"poly3table"<-function(time, status, f, tumour=NULL, symbol="*")
{

N<-length(time)

if(N!=length(status))
{stop("time and status must be vectors of equal length")}

if(length(symbol)!=1)
  {stop("symbol must be a single value")}
 
statussign<-character(length=N)

if(is.null(tumour))
 {
  if(is.logical(status))
   {
   wtumour<-which(status==TRUE)
   statussign[wtumour]<-symbol
   statussign[-wtumour]<-""
   }
  else{

  if(!is.logical(status))
    {stop("status must be a logical vector;
      if not, please specify the value specifying presence of tumour
      in argument tumour")}
    }
 }
else
 {
 
  if(length(tumour)!=1)
   {stop("tumour must be a single value")}

  wtumor<-which(status==tumour)
  if(length(wtumor)==0)
   {warning("tumor was not found in status")}

  statussign[wtumor]<-symbol
  statussign[-wtumor]<-""
 }

outvec<-paste(time, statussign, sep="")

datlist<-split(outvec, f=as.factor(f))

shortendat<-function(x)
 {
  tab<-table(x)
  nam<-names(tab)
  ntab<-as.numeric(tab)
  wg1<-which(ntab>1)
  nchar<-as.character(ntab)
  nchar2<-paste("(", nchar,")", sep="")
  nchar2[-wg1]<-""
  nchar3<-paste(paste(nam, nchar2, sep=""), collapse=", ")
  return(nchar3)
 }

datlist2<-lapply(X=datlist, FUN=function(x){shortendat(x)})

return(datlist2)

}

`print.binomORci` <-
function(x,...)
{

# A table of confidence intervals

aargs<-list(...)

if(is.null(aargs$digits))
 {digits<-4}
else
 {digits<-aargs$digits}

dist<-attr(x$quantile, which="dist")

if(dist=="MVN"){
cat("Simultaneous", round(x$conf.level*100,3),"percent-confidence intervals","\n",
"for the odds ratio (OR)", "\n")
}
else{
cat("Local", round(x$conf.level*100,3),"percent-confidence intervals","\n",
"for the odds ratio (OR)", "\n")
}

conf.int <- cbind(x$estimate, x$conf.int)

colnames(conf.int)[1] <- "estimate"

print(round(conf.int, digits=digits))

ORdef<-paste("p(",x$success,")/(1-p(",x$success,"))", sep="")

cat(" ","\n")
cat("where the odds is defined:", ORdef, "\n")
cat(" ","\n")

invisible(x)
}

powerbinom <-
function(p, n, alpha=0.05, type="Dunnett", cmat=NULL, rhs=0, alternative=c("two.sided", "less", "greater"), ptype=c("global", "anypair", "allpair"), method="Wald", crit=NULL, ...)
{

if(any(c(n*p, n*(1-p))<5)){warning("For small n and extreme proportions this function might give misleading approximations.")}

if(length(alpha)!=1 || !is.numeric(alpha) | alpha<=0 | alpha>0.5){stop("alpha must be a single numeric value between 0 and 0.5")}
 ptype <- match.arg(ptype)
 alternative <- match.arg(alternative)
if( length(n)<2 || !(is.numeric(n)|is.integer(n)) ){stop("n must be a vector of sample sizes (at least length 2, containing numeric or integer values)")}
if( length(p)<2 || !is.numeric(p) || (any(p<0)| any(p>1)) ){stop("p must be a vector of proportions (at least length 2, containing numeric values between 0 and 1)")}
ngroup<-length(p)
if(length(n)!=ngroup){stop("length of vector n must equal the length of vector p")}

if(is.null(cmat)){cmat<-contrMat(n=n, type=type)}else{
if(!is.matrix(cmat) | !is.numeric(cmat)){stop("cmat must be a matrix with numeric entries")}
if(ncol(cmat)!=ngroup){stop("number of columns in cmat must equal the length of p")}
#        if(nrow(cmat)<1){stop("number of rows (i.e. number of comparisons) in cmat should be at least 2")}
}

M <- nrow(cmat)

if(!is.numeric(rhs) & !is.integer(rhs)){stop("rhs must be a (vector of) numeric value(s).")}
if(any(rhs >= 1) | any(rhs <= (-1))){stop("rhs is bounded between -1 and 1 for differences of proportions.")}
if(alternative=="two.sided" & any(rhs != 0)){warning("Power of shifted tests (i.e. rhs!=0) makes sense for one-sided alternatives, but alternative='two.sided' has been specified.")}
RHS<-rep(rhs, length.out=M)

switch(method,
"Wald"={
  Ltrue <- as.numeric(cmat%*%p)
  LExpectH1 <- cmat%*%p
  VarpH1 <- p*(1-p)/n
  LVarH1 <- (cmat^2) %*% (VarpH1)
  ExpTH1 <-(LExpectH1-RHS)/sqrt(LVarH1)
 },
"ADD1"={
  Ltrue <- as.numeric(cmat%*%p)
  PExpH1<-(p*n+0.5)/(n+1)
  LExpectH1 <- cmat%*%PExpH1
  VarpH1 <- PExpH1*(1-PExpH1)/(n+1)
  LVarH1 <- (cmat^2) %*% (VarpH1)
  ExpTH1 <- (LExpectH1-RHS)/sqrt(LVarH1)
 },
"ADD2"={
  Ltrue <- as.numeric(cmat%*%p)
  PExpH1<-(p*n+1)/(n+2)
  LExpectH1 <- cmat%*%PExpH1
  VarpH1 <- PExpH1*(1-PExpH1)/(n+2)
  LVarH1 <- (cmat^2) %*% (VarpH1)
  ExpTH1 <- (LExpectH1-RHS)/sqrt(LVarH1)
 })

# calculate the correlation matrix

CORR <- corrMatgen(CM=cmat, varp=VarpH1)

if(is.null(crit)){
switch(alternative,
two.sided={crit <- qmvnorm(p=1-alpha, tail =  "both.tails", sigma = CORR, ...)[["quantile"]]},
less={crit <- qmvnorm(p=1-alpha, tail =  "upper",  sigma = CORR, ...)[["quantile"]]},
greater={crit <- qmvnorm(p=1-alpha, tail =  "lower", sigma = CORR,...)[["quantile"]]})

}else{
if(!is.numeric(crit)| length(numeric)!=1){stop("crit must be a single numeric value")}
switch(alternative,
two.sided={if(crit<0){crit <- abs(crit); warning("Specified critical value 'crit' is negative, its absolute value will be used instead.")}},
less={if(crit>0){crit <- (-1)*crit; warning("Specified critical value 'crit' is positive, its negative value will be used instead.")}},
greater={if(crit<0){crit <- abs(crit); warning("Specified critical value 'crit' is negative, its absolute value will be used instead.")}})
}

ExpTeststat<-as.numeric(ExpTH1)
M<-length(ExpTeststat)

switch(EXPR=ptype,
"global"={

switch(EXPR=alternative,
"two.sided"={beta <- pmvnorm(lower=rep(-crit,M), upper=rep(crit,M), mean=ExpTeststat, sigma=CORR,...); whichHA <- which(abs(Ltrue-RHS) > 10*.Machine$double.eps)},
"less"={beta <- pmvnorm(lower=rep(crit,M), upper=rep(Inf, M), mean=ExpTeststat, sigma=CORR,...); whichHA <- which(Ltrue-RHS < (-10)*.Machine$double.eps)},
"greater"={beta <- pmvnorm(lower=rep(-Inf,M), upper=rep(crit,M), mean=ExpTeststat, sigma=CORR,...); whichHA <- which(Ltrue-RHS > 10*.Machine$double.eps)})
},

"anypair"={

switch(EXPR=alternative,
"two.sided"={
whichHA <- which(abs(Ltrue-RHS) > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated."); beta <- 1-alpha}else{
beta <- pmvnorm(lower=rep(-crit, MHA), upper=rep(crit, MHA), mean=ExpTeststat[whichHA], sigma=CORR[whichHA,whichHA],...)}
},
"less"={
whichHA <- which(Ltrue-RHS < (-10)*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated."); beta <- 1-alpha}else{
beta <- pmvnorm(lower=rep(crit, MHA), upper=rep(Inf, MHA), mean=ExpTeststat[whichHA], sigma=CORR[whichHA,whichHA],...)}
},
"greater"={
whichHA <- which(Ltrue-RHS > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated."); beta <- 1-alpha}else{
beta <- pmvnorm(lower=rep(-Inf,MHA), upper=rep(crit, MHA), mean=ExpTeststat[whichHA], sigma=CORR[whichHA,whichHA],...)}
})
},

"allpair"={

switch(EXPR=alternative,
"two.sided"={
whichHA <- which(abs(Ltrue-RHS) > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated."); beta<-1-alpha}else{
nsim <- 10000
RT <- rmvnorm(n=nsim, mean=ExpTeststat[whichHA], sigma=CORR[whichHA, whichHA], method="svd")
nreject <- sum( apply(RT, 1, function(x){min(abs(x))}) > abs(crit) )
beta <- 1 - (nreject/nsim)
simerror <- sqrt(beta*(1-beta)/nsim)
attr(beta, which="simerror")<-simerror
}},
"less"={
whichHA <- which(Ltrue-RHS < (-10)*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated."); beta<-1-alpha}else{
beta <- 1 - pmvnorm(lower=rep(-Inf, MHA), upper=rep(crit,MHA), mean=ExpTeststat[whichHA], sigma=CORR[whichHA, whichHA],...)
}},
"greater"={
whichHA <- which(Ltrue-RHS > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated."); beta<-1-alpha}else{
beta <- 1 - pmvnorm(lower=rep(crit,MHA), upper=rep(Inf,MHA), mean=ExpTeststat[whichHA],  sigma=CORR[whichHA, whichHA],...)
}})
})

pow<- (1-beta)

HAtrue<-numeric(length=M)
HAtrue[whichHA]<-1


if(is.null(colnames(cmat))){colnames(cmat)<-paste("g", 1:ngroup, sep="")}
settings<-data.frame(cmat, trueContrast=Ltrue, rhs=RHS, ExpTstat=ExpTeststat, underHA=HAtrue)

RES<-list(power=pow,
n=n,
p=p,
conexp=settings,
crit=crit,
alternative=alternative,
ptype=ptype,
alpha=alpha
)

return(RES)
}



powerbinomOR <- function (p, n, alpha = 0.05, type = "Dunnett", cmat = NULL, 
    rhs = 1, alternative = c("two.sided", "less", "greater"), 
    ptype = c("global", "anypair", "allpair"), 
    crit = NULL, ...) 
{
    if (any(c(n * p, n * (1 - p)) < 5)) {
        warning("For small n and extreme proportions this function might give misleading approximations.")
    }
    if (length(alpha) != 1 || !is.numeric(alpha) | alpha <= 0 | 
        alpha > 0.5) {
        stop("alpha must be a single numeric value between 0 and 0.5")
    }
    ptype <- match.arg(ptype)
    alternative <- match.arg(alternative)
    if (length(n) < 2 || !(is.numeric(n) | is.integer(n))) {
        stop("n must be a vector of sample sizes (at least length 2, containing numeric or integer values)")
    }
    if (length(p) < 2 || !is.numeric(p) || (any(p <= 0) | any(p >=
        1))) {
        stop("p must be a vector of proportions (at least length 2, containing numeric values greater than 0 and smaller than 1)")
    }
    ngroup <- length(p)
    if (length(n) != ngroup) {
        stop("length of vector n must equal the length of vector p")
    }
    if (is.null(cmat)) {
        cmat <- contrMat(n = n, type = type)
    }
    else {
        if (!is.matrix(cmat) | !is.numeric(cmat)) {
            stop("cmat must be a matrix with numeric entries")
        }
        if (ncol(cmat) != ngroup) {
            stop("number of columns in cmat must equal the length of p")
        }
    }
    M <- nrow(cmat)
    if (!is.numeric(rhs) & !is.integer(rhs)) {
        stop("rhs must be a (vector of) numeric value(s).")
    }
    if (any(rhs <= 0)) {
        stop("rhs must be greater 0 for odds ratios.")
    }
    if (alternative == "two.sided" & any(rhs != 1)) {
        warning("Power of shifted tests (i.e. rhs!=1) makes sense for one-sided alternatives, but alternative='two.sided' has been specified.")
    }
    RHS <- rep(rhs, length.out = M)
	LRHS <- log(RHS)
	logodds <- log(p/(1-p))
        Ltrue <- as.numeric(cmat %*% logodds)
        LExpectH1 <- cmat %*% logodds
        VarpH1 <- 1/(p*n) + 1/((1-p)*n)
        LVarH1 <- (cmat^2) %*% (VarpH1)
        ExpTH1 <- (LExpectH1 - LRHS)/sqrt(LVarH1)
 
    CORR <- corrMatgen(CM = cmat, varp = VarpH1)
    if (is.null(crit)) {
        switch(alternative, two.sided = {
            crit <- qmvnorm(p = 1 - alpha, tail = "both.tails", 
                sigma = CORR, ...)[["quantile"]]
        }, less = {
            crit <- qmvnorm(p = 1 - alpha, tail = "upper", sigma = CORR, 
                ...)[["quantile"]]
        }, greater = {
            crit <- qmvnorm(p = 1 - alpha, tail = "lower", sigma = CORR, 
                ...)[["quantile"]]
        })
    }
    else {
        if (!is.numeric(crit) | length(numeric) != 1) {
            stop("crit must be a single numeric value")
        }
        switch(alternative, two.sided = {
            if (crit < 0) {
                crit <- abs(crit)
                warning("Specified critical value 'crit' is negative, its absolute value will be used instead.")
            }
        }, less = {
            if (crit > 0) {
                crit <- (-1) * crit
                warning("Specified critical value 'crit' is positive, its negative value will be used instead.")
            }
        }, greater = {
            if (crit < 0) {
                crit <- abs(crit)
                warning("Specified critical value 'crit' is negative, its absolute value will be used instead.")
            }
        })
    }
    ExpTeststat <- as.numeric(ExpTH1)
    M <- length(ExpTeststat)
    switch(EXPR = ptype, global = {
        switch(EXPR = alternative, two.sided = {
            beta <- pmvnorm(lower = rep(-crit, M), upper = rep(crit, 
                M), mean = ExpTeststat, sigma = CORR, ...)
            whichHA <- which(abs(Ltrue - LRHS) > 10 * .Machine$double.eps)
        }, less = {
            beta <- pmvnorm(lower = rep(crit, M), upper = rep(Inf, 
                M), mean = ExpTeststat, sigma = CORR, ...)
            whichHA <- which(Ltrue < LRHS)
        }, greater = {
            beta <- pmvnorm(lower = rep(-Inf, M), upper = rep(crit, 
                M), mean = ExpTeststat, sigma = CORR, ...)
            whichHA <- which(Ltrue > LRHS)
        })
    }, anypair = {
        switch(EXPR = alternative, two.sided = {
            whichHA <- which(abs(Ltrue - LRHS) > 10 * .Machine$double.eps)
            MHA <- length(whichHA)
            if (MHA < 1) {
                warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated.")
                beta <- 1 - alpha
            } else {
                beta <- pmvnorm(lower = rep(-crit, MHA), upper = rep(crit, 
                  MHA), mean = ExpTeststat[whichHA], sigma = CORR[whichHA, 
                  whichHA], ...)
            }
        }, less = {
            whichHA <- which(Ltrue < LRHS)
            MHA <- length(whichHA)
            if (MHA < 1) {
                warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated.")
                beta <- 1 - alpha
            } else {
                beta <- pmvnorm(lower = rep(crit, MHA), upper = rep(Inf, 
                  MHA), mean = ExpTeststat[whichHA], sigma = CORR[whichHA, 
                  whichHA], ...)
            }
        }, greater = {
            whichHA <- which(Ltrue > LRHS)
            MHA <- length(whichHA)
            if (MHA < 1) {
                warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated.")
                beta <- 1 - alpha
            } else {
                beta <- pmvnorm(lower = rep(-Inf, MHA), upper = rep(crit, 
                  MHA), mean = ExpTeststat[whichHA], sigma = CORR[whichHA, 
                  whichHA], ...)
            }
        })
    }, allpair = {
        switch(EXPR = alternative, two.sided = {
            whichHA <- which(abs(Ltrue - LRHS) > 10 * .Machine$double.eps)
            MHA <- length(whichHA)
            if (MHA < 1) {
                warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated.")
                beta <- 1 - alpha
            } else {
                nsim <- 10000
                RT <- rmvnorm(n = nsim, mean = ExpTeststat[whichHA], 
                  sigma = CORR[whichHA, whichHA], method = "svd")
                nreject <- sum(apply(RT, 1, function(x) {
                  min(abs(x))
                }) > abs(crit))
                beta <- 1 - (nreject/nsim)
                simerror <- sqrt(beta * (1 - beta)/nsim)
                attr(beta, which = "simerror") <- simerror
            }
        }, less = {
            whichHA <- which(Ltrue < LRHS)
            MHA <- length(whichHA)
            if (MHA < 1) {
                warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated.")
                beta <- 1 - alpha
            } else {
                beta <- 1 - pmvnorm(lower = rep(-Inf, MHA), upper = rep(crit, 
                  MHA), mean = ExpTeststat[whichHA], sigma = CORR[whichHA, 
                  whichHA], ...)
            }
        }, greater = {
            whichHA <- which(Ltrue > LRHS)
            MHA <- length(whichHA)
            if (MHA < 1) {
                warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated.")
                beta <- 1 - alpha
            } else {
                beta <- 1 - pmvnorm(lower = rep(crit, MHA), upper = rep(Inf, 
                  MHA), mean = ExpTeststat[whichHA], sigma = CORR[whichHA, 
                  whichHA], ...)
            }
        })
    })
    pow <- (1 - beta)
    HAtrue <- numeric(length = M)
    HAtrue[whichHA] <- 1
    if (is.null(colnames(cmat))) {
        colnames(cmat) <- paste("g", 1:ngroup, sep = "")
    }
    settings <- data.frame(cmat, trueOddsRatio = exp(Ltrue), rhs = RHS, 
        trueLogit=Ltrue, rhsLogitscale=LRHS,
        ExpTstat = ExpTeststat, underHA = HAtrue)
    RES <- list(power = pow, n = n, p = p, conexp = settings, 
        crit = crit, alternative = alternative, ptype = ptype, 
        alpha = alpha)
    return(RES)
}

`binomORci` <-
function(x, ...){UseMethod("binomORci")}


`binomORci.default` <-
function(x, n, names=NULL, type="Dunnett", method="GLM",
 cmat=NULL, alternative="two.sided", conf.level=0.95, dist="MVN", ...)
{

aargs<-list(...)

method<-match.arg(method, choices=c("GLM", "Woolf"))

# check x, n

    k <- length(x)
    if(k<2)
     {stop("x, n must contain at least two elements")}

    if(any(n<2)) 
     {stop("sample size must be at least 2 in each group")}   

    if(length(n)!=k)
     {stop("x, n must be of equal length")}

    if(!is.numeric(x) | !is.numeric(n))
     {stop("x, n must be numeric vectors")}



# check the names
# if no names are given, available names of x are taken
# if names are specified, names of x/n are overwritten

    gnames<-names

    if(is.null(gnames))
     {
      if(is.null(names(x)))
       {gnames <- names(x) <- names(n) <- as.character(1:k)}
       else
        {gnames <- names(n) <- names(x)}
     }
     else
      {
       if(length(gnames)!=k)
        {stop("length of names does not fit to length of x,n")}
        else
         {names(x) <- names(n) <- as.character(gnames)}
      }

# check the contrast matrix

    if (is.null(cmat)) {
      if(type=="Dunnett") {
        if(is.null(aargs$base)){base<-1}
        else{base<-aargs$base}
        cmat <- contrMat(n=n, type=type, base=base)
       }
       else{cmat <- contrMat(n = n, type = type)}
    }
    else {
        if (!is.matrix(cmat) || ncol(cmat) != k)
         {stop("cmat must be a matrix with number of columns = number of groups")}
	attr(cmat, which="type")<-"User defined"
    }


# get the point and variance estimates

switch(method,
GLM={
    est <- binomest.default(x=x, n=n, names=gnames, success=aargs$success)
    grp<-factor(est$names, levels=est$names)
    logisticfit <- glm(cbind(x,n-x) ~ 0 + grp, family=binomial(link="logit"))
    eta <- coefficients(logisticfit)
    sigma <- vcov(logisticfit)

    out<-Waldci( cmat=cmat,
     estp=eta,
     varp=diag(sigma),
     varcor=diag(sigma),
     alternative = alternative,
     conf.level=conf.level,
     dist=dist)


    conf.int <- exp(out$conf.int)
    estimate <- exp(cmat %*% eta)

    if(!is.null(dimnames(cmat)[[1]]))
    {
    cnamesD<-dimnames(cmat)[[1]]
    cnamesSlist<-strsplit(cnamesD, "-")
    cnamesOR<-unlist(lapply(cnamesSlist, FUN=function(x){paste(x, collapse="/")}))
    }
    else{cnamesOR<-paste("C",1:nrow(cmat),sep="")}

    rownames(estimate)<-cnamesOR
    rownames(conf.int)<-cnamesOR    

    out$conf.int<-conf.int
    out$estimate<-estimate
    colnames(cmat) <- gnames
    out$x <- x
    out$n <- n
    out$p <- est$estimate
    out$success <- est$success
    out$names <- gnames
    out$method <- "glm"
    out$cmat <- cmat
},
Woolf={

# Restrict Bounds to certain values, when x=0 or x=n occurs

restrictboundsOR<-function(x, n, cmat, conf.int)
{

if(all(x!=0) & all(x!=n) )
 {return(conf.int)}
else{

M<-nrow(cmat)

warning("0 occured in the data and the risk ratio might not be defined")

for(i in 1:M)
{

# 'Uninformative events'
# all x=0 in the ith contrast:

cNOT0<-sign(cmat[i,])!=0



if( all( x[cNOT0]==0 ) )
 {
  conf.int[i,1]<-0
  conf.int[i,2]<-Inf
#  cat("Contrast",i,"All x=0 \n")
 }

# all x=n in the ith contrast:

if( all( x[cNOT0]==n[cNOT0] ) )
 {
  conf.int[i,1]<-0
  conf.int[i,2]<-Inf
# cat("Contrast",i,"All x=n \n")
 }


# all numerator x=0 in contrast i

cINDN<-sign(cmat[i,])==1

if(all ( x[cINDN]==0 ) )
 {
 conf.int[i,1]<-0
# cat("Numerator Contrast",i,"All x=0 \n")
 }


# all numerator x=n in contrast i

if(all ( x[cINDN]==n[cINDN] ) )
 {
  conf.int[i,2] <- Inf
 # cat("Numerator Contrast",i,"All x=n \n")
 }


# all denominator x=0

cINDD<-sign(cmat[i,])==(-1)

if(all (x[cINDD]==0 ) )
 {
  conf.int[i,2]<-Inf
 # cat("Denominator Contrast",i,": all x=0 \n")
 }

# all denominator x=n

if(all (x[cINDD]==n[cINDD] ) )
 {
  conf.int[i,1] <- 0 
 # cat("Denominator Contrast",i,": all x=n \n")
 }


}

return(conf.int)
}

}


    est <- binomest.default(x=x, n=n, names=gnames, success=aargs$success)
    
    XPI <- est$Y+0.5 
    XQI <- est$n-est$Y+0.5

    estI <- log(XPI/XQI)

    varI <- 1/XPI + 1/XQI

    out<-Waldci( cmat=cmat,
     estp=estI,
     varp=varI,
     varcor=varI,
     alternative = alternative,
     conf.level=conf.level,
     dist=dist)


    conf.int <- exp(out$conf.int)
    estimate <- exp(cmat %*% estI)

    if(!is.null(dimnames(cmat)[[1]]))
    {
    cnamesD<-dimnames(cmat)[[1]]
    cnamesSlist<-strsplit(cnamesD, "-")
    cnamesOR<-unlist(lapply(cnamesSlist, FUN=function(x){paste(x, collapse="/")}))
    }
    else{cnamesOR<-paste("C",1:nrow(cmat),sep="")}

    rownames(estimate)<-cnamesOR
    rownames(conf.int)<-cnamesOR    
 
    conf.int<-restrictboundsOR(x=x, n=n, cmat=cmat, conf.int=conf.int)

    out$conf.int<-conf.int
    out$estimate<-estimate
    colnames(cmat) <- gnames
    out$x <- x
    out$n <- n
    out$p <- est$estimate
    out$success <- est$success
    out$names <- gnames
    out$method <- "Adjusted Woolf"
    out$cmat <- cmat
})




    class(out) <- c("binomORci", "sci")
    return(out)
}


`binomORci.formula` <-
function(formula, data, type="Dunnett", method="GLM",
 cmat=NULL, alternative="two.sided", conf.level=0.95, dist="MVN", ...)
{
aargs<-list(...)

est<-binomest.formula(formula=formula, data=data, success=aargs$success)

aargs$x<-est$Y
aargs$n<-est$n
aargs$names<-est$names
aargs$type<-type
aargs$method<-method
aargs$cmat<-cmat
aargs$alternative<-alternative
aargs$conf.level<-conf.level
aargs$success<-est$success
aargs$dist<-dist

out<-do.call("binomORci.default", args=aargs)

return(out)

}




`binomORci.table` <-
function(x, type="Dunnett", method="GLM",
 cmat=NULL, alternative="two.sided", conf.level=0.95, dist="MVN", ...)
{
aargs<-list(...)

est<-binomest.table(x=x, success=aargs$success)

aargs$x<-est$Y
aargs$n<-est$n
aargs$names<-est$names
aargs$type<-type
aargs$method<-method
aargs$cmat<-cmat
aargs$alternative<-alternative
aargs$conf.level<-conf.level
aargs$success<-est$success
aargs$dist<-dist

out<-do.call("binomORci.default", args=aargs)

return(out)

}

`binomORci.matrix` <-
function(x, type="Dunnett", method="GLM",
 cmat=NULL, alternative="two.sided", conf.level=0.95, dist="MVN", ...)
{
aargs<-list(...)

tab<-as.table(x)

est<-binomest.table(x=tab, success=aargs$success)

aargs$x<-est$Y
aargs$n<-est$n
aargs$names<-est$names
aargs$type<-type
aargs$method<-method
aargs$cmat<-cmat
aargs$alternative<-alternative
aargs$conf.level<-conf.level
aargs$success<-est$success
aargs$dist<-dist

out<-do.call("binomORci.default", args=aargs)

return(out)

}

powermcpt <-
function(mu, n, sd, cmat=NULL, rhs=0, type="Dunnett", alternative=c("two.sided", "less", "greater"), alpha=0.05, ptype=c("global", "anypair", "allpair"), crit=NULL, ...)
{

rmvtFS <- function (n, sigma = diag(2), df = 1, delta = rep(0, nrow(sigma)), 
    type = c("shifted", "Kshirsagar"), method=c("eigen", "svd", "chol")) 
{
    if (length(delta) != nrow(sigma)) 
        stop("delta and sigma have non-conforming size")
    if (df == 0) 
        return(rmvnorm(n, mean = delta, sigma = sigma, method=method))
    type <- match.arg(type)
    if (type == "Kshirsagar") 
        return(rmvnorm(n, mean = delta, sigma = sigma, method=method)/sqrt(rchisq(n, 
            df)/df))
    if (type == "shifted") {
        sims <- rmvnorm(n, sigma = sigma, method=method)/sqrt(rchisq(n, df)/df)
        return(sweep(sims, 2, delta, "+"))
    }
}

if(length(sd)!=1 || !is.numeric(sd)){stop("sd must be a single numeric value")}
if(length(alpha)!=1 || !is.numeric(alpha) | alpha<=0 | alpha>0.5){stop("alpha must be a single numeric value between 0 and 0.5")}
 ptype <- match.arg(ptype)
 alternative <- match.arg(alternative)
if( length(n)<2 || !(is.numeric(n)|is.integer(n)) ){stop("n must be a vector of sample sizes (at least length 2, containing numeric or integer values)")}
if( length(mu)<2 || !(is.numeric(mu)|is.integer(mu)) ){stop("mu must be a vector of expected means (at least length 2, containing numeric or integer values)")}
        ngroup<-length(mu)
        if(length(n)!=ngroup){stop("length of vector n must equal the length of vector mu")}

if(is.null(cmat)){cmat<-contrMat(n=n, type=type)}else{
if(!is.matrix(cmat) | !is.numeric(cmat))stop("cmat must be a matrix with numeric entries")
        if(ncol(cmat)!=ngroup){stop("number of columns in cmat must equal the length of mu")}
        if(nrow(cmat)<1){stop("number of rows (i.e. number of comparisons) in cmat should be at least 2")}
}

if(!is.numeric(rhs) & !is.integer(rhs)){stop("rhs must be a (vector of) numeric value(s).")}

M <- nrow(cmat)
MU <- matrix(mu, ncol=1)
VCOV <- diag((sd^2)/n)
VCOVL <- cmat %*% VCOV %*% t(cmat)
CORR <- cov2cor(VCOVL)
dfR <- sum(n)-length(n)

RHS<-rep(rhs, length.out=M)


if(is.null(crit)){
switch(alternative,
two.sided={crit <- qmvt(p=1-alpha,tail =  "both.tails", df = dfR, corr = CORR, ...)[["quantile"]]},
less={crit <- qmvt(p=1-alpha,tail =  "upper", df = dfR,  corr = CORR, ...)[["quantile"]]},
greater={crit <- qmvt(p=1-alpha,tail =  "lower", df = dfR, corr = CORR,...)[["quantile"]]})

}else{
if(!is.numeric(crit)| length(numeric)!=1){stop("crit must be a single numeric value")}
switch(alternative,
two.sided={if(crit<0){crit <- abs(crit); warning("Specified critical value 'crit' is negative, its absolute value will be used instead.")}},
less={if(crit>0){crit <- (-1)*crit; warning("Specified critical value 'crit' is positive, its negative value will be used instead.")}},
greater={if(crit<0){crit <- abs(crit); warning("Specified critical value 'crit' is negative, its absolute value will be used instead.")}})
}


ExpL <- cmat %*% MU
Ltrue <- as.numeric(ExpL)
ExpVL <- diag(VCOVL)
ExpTeststat <- (as.numeric(ExpL)-RHS)/sqrt(as.numeric(ExpVL))

switch(EXPR=ptype,
"global"={

switch(EXPR=alternative,
"two.sided"={beta <- pmvt(lower=rep(-crit,M), upper=rep(crit,M), delta=ExpTeststat, df=dfR, corr=CORR,...); whichHA <- which(abs(Ltrue-RHS) > 10*.Machine$double.eps)},
"less"={beta <- pmvt(lower=rep(crit,M), upper=rep(Inf, M), delta=ExpTeststat, df=dfR, corr=CORR,...); whichHA <- which(Ltrue-RHS < (-10)*.Machine$double.eps)},
"greater"={beta <- pmvt(lower=rep(-Inf,M), upper=rep(crit,M), delta=ExpTeststat, df=dfR, corr=CORR,...); whichHA <- which(Ltrue-RHS > 10*.Machine$double.eps)})
},

"anypair"={

switch(EXPR=alternative,
"two.sided"={
whichHA <- which(abs(Ltrue-RHS) > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated."); beta <- 1-alpha}else{
beta <- pmvt(lower=rep(-crit, MHA), upper=rep(crit, MHA), delta=ExpTeststat[whichHA], df=dfR, corr=CORR[whichHA,whichHA],...)}
},
"less"={
whichHA <- which(Ltrue-RHS < (-10)*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated."); beta <- 1-alpha}else{
beta <- pmvt(lower=rep(crit, MHA), upper=rep(Inf, MHA), delta=ExpTeststat[whichHA], df=dfR, corr=CORR[whichHA,whichHA],...)}
},
"greater"={
whichHA <- which(Ltrue-RHS > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under their corresponding null hypothesis, anypair power can not be calculated."); beta <- 1-alpha}else{
beta <- pmvt(lower=rep(-Inf,MHA), upper=rep(crit, MHA), delta=ExpTeststat[whichHA], df=dfR, corr=CORR[whichHA,whichHA],...)}
})
},

"allpair"={

switch(EXPR=alternative,
"two.sided"={
whichHA <- which(abs(Ltrue-RHS) > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated."); beta<-1-alpha}else{
nsim <- 10000
RT <- rmvtFS(n=nsim, delta=ExpTeststat[whichHA], df=dfR, sigma=CORR[whichHA, whichHA], method="svd")
nreject <- sum( apply(RT, 1, function(x){min(abs(x))}) > abs(crit) )
beta <- 1 - (nreject/nsim)
simerror <- sqrt(beta*(1-beta)/nsim)
attr(beta, which="simerror")<-simerror
}},
"less"={
whichHA <- which(Ltrue-RHS < (-10)*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated."); beta<-1-alpha}else{
beta <- 1 - pmvt(lower=rep(-Inf, MHA), upper=rep(crit,MHA), delta=ExpTeststat[whichHA], df=dfR, corr=CORR[whichHA, whichHA],...)
}},
"greater"={
whichHA <- which(Ltrue-RHS > 10*.Machine$double.eps)
MHA <- length(whichHA)
if(MHA<1){warning("All contrasts are under the corresponding null hypotheses, all pair power can not be calculated."); beta<-1-alpha}else{
beta <- 1 - pmvt(lower=rep(crit,MHA), upper=rep(Inf,MHA), delta=ExpTeststat[whichHA], df=dfR, corr=CORR[whichHA, whichHA],...)
}})
})


pow<- (1-beta)

if(is.null(colnames(cmat))){colnames(cmat)<-paste("g", 1:ngroup, sep="")}

HAtrue<-numeric(length=M)
HAtrue[whichHA]<-1

settings<-data.frame(cmat, expContrast=ExpL, rhs=RHS, ExpTstat=ExpTeststat, underHA=HAtrue)

out<-list(power=pow,
mu=mu, n=n,
conexp=settings,
crit=crit,
alternative=alternative,
ptype=ptype,
alpha=alpha
)

return(out)
}

`print.poly3ci` <-
function(x, digits=4, ...)
{


if(x$method=="BP")
 {methvar<-"Bailer-Portier"}
if(x$method=="BW")
 {methvar<-"Bieler-Williams"}
if(x$method=="ADD1")
 {methvar<-"Add-1"}
if(x$method=="ADD2")
 {methvar<-"Add-2"}


#  Table of sample estimates

sample.estimate<-rbind(x$sample.estimate$Y, x$sample.estimate$n, x$sample.estimate$nadj, x$sample.estimate$estimate)
rownames(sample.estimate)<-c("x","n", "adjusted n", "adjusted estimate")

cat("Sample estimates, using poly-",x$k,"-adjustment","\n")
print(round(sample.estimate, digits=digits))
cat(" ","\n")

# The contrast matrix:

cat("Contrast matrix:","\n")
print(x$cmat)
cat(" ","\n")

# A table of confidence intervals


dist<-attr(x$quantile, which="dist")

if(dist=="MVN"){
cat("Simultaneous",x$conf.level*100,"percent confidence intervals using",methvar,"variance estimators:","\n")
}
else{
cat("Local",x$conf.level*100,"percent confidence intervals using",methvar,"variance estimators:","\n")
}

conf.int <- cbind(x$estimate, x$conf.int)

colnames(conf.int)[1] <- "estimate"

print(round(conf.int, digits=digits))

cat(" ","\n")

invisible(x)

}

`poly3test` <-
function(time, status, f, type="Dunnett", cmat=NULL, method="BP", alternative="two.sided", dist="MVN", k=3, ...)

{

aargs<-list(...)

est <- poly3estf(time=time, status=status, f=f, tmax=max(time), method=method, k=k)

n <- est$n
varnames<-est$names
ngroups <- length(n)


# check the contrast matrix

    if (is.null(cmat)) {
      if(type=="Dunnett") {
        if(is.null(aargs$base)){base<-1}
        else{base<-aargs$base}
        cmat <- contrMat(n=n, type=type, base=base)
       }
       else{cmat <- contrMat(n = n, type = type)}
    }
    else {
        if (!is.matrix(cmat) || ncol(cmat) != ngroups)
         {stop("cmat must be a matrix with number of columns = number of groups")}
    }

# Check margin

out <- Waldtest(estp=est$estp, varp=est$varcor, cmat=cmat, alternative=alternative, dist=dist)

# estimates using the MLE

out$estimate <- cmat %*% est$estimate


colnames(cmat)<-varnames

out$time<-time
out$status<-status
out$f<-f
out$method<-method
out$cmat<-cmat
out$k<-k

out$sample.estimate<-est

class(out)<-"poly3test"

return(out)

}

'binomest'<-function(x, ...){UseMethod("binomest")}

'binomest.default' <-
function(x, n, names=NULL, method="Wald", success=NULL, ...)
{

if(is.null(success))
 {success<-"success"}
else
 {success<-paste(as.character(success), collapse="")}

method<-match.arg(arg = method,
      choices = c("Wald", "ADD1", "ADD2"))

Ntot <- sum(n)
k <- length(n)


if(is.null(names))
 {names<-paste(1:k)}


if(length(x) != k || length(names)!= k)
 {stop("arguments x, n, and names must be vectors of equal length")}

Y<-x

estimate <- Y/n

if(method=="Wald")
{ estp <- estimate
  varp <- estp*(1-estp)/n

 which0<-which(Y==0)
 whichn<-which(Y==n)

 estcor <- estp
 estcor[which0] <- 0.5/n[which0]
 estcor[whichn] <- (n[whichn]-0.5)/n[whichn]

 varcor <- estcor*(1-estcor)/n
}

if(method=="ADD1")
{
  estp <- (Y+0.5)/(n+1)
  varp <- estp*(1-estp)/(n+1)
  varcor <- varp
}

if(method=="ADD2")
{
  estp <- (Y+1)/(n+2)
  varp <- estp*(1-estp)/(n+2)
  varcor <- varp
}


names(Y) <- names(n) <- names(estimate) <- names(estp) <- names(varp) <- names(varcor) <- names

return(list(
Y=Y,
n=n,
estimate=estimate,
estp=estp,
varp=varp,
varcor=varcor,
names=names,
success=success
))

}




'binomest.formula' <- function(formula, data, method="Wald", success=NULL, ...)
{

if(length(formula)!=3)
 {stop("formula must be of shape 'response ~ treatment'") }

mf<-model.frame(formula=formula, data=data)

mf[,2]<-as.factor(mf[,2])

mf[,1]<-as.factor(mf[,1])

tab<-table(mf)

 if(length(dimnames(tab)[[1]])!=2)
  {stop("The specified response variable should have two levels")}


if(is.null(success))
 {

  if(length(dimnames(tab)[[1]])!=2)
   {warning("The response variable has more than 2 levels. The first will be taken as success")}

  n<-colSums(tab)
  x<-tab[1,]
  names<-dimnames(tab)[[2]]
  success<-dimnames(tab)[[1]][1]
 }
else
 {
  success<-as.character(success)
  if(length(success)!=1)
   {stop("Argument success should be a single character string,
 naming one of the levels of the response variable")}

  if(all(dimnames(tab)==success))
   {stop(paste("Level", success, "could not be found in the specified response variable"))}

 n<-colSums(tab)
 x<-tab[success,]
 names<-dimnames(tab)[[2]]
 }


out <- binomest.default(x=x, n=n, names=names,
 method=method, success=success)

return(out)
}



'binomest.table' <-
function(x, method="Wald", success=NULL, ...)
{

if(length(dim(x))!=2)
 {stop("x must be a table with two dimensions")}

if(!any(dim(x)==2))
 {stop("None dimension of x equals 2, this function can only handle 2xk tables")}

respdim<-which(dim(x)==2)

groupdim<-which(dim(x)>=2)

if(respdim==1)
 {
  if(is.null(success))
   {
   resp <- as.numeric(x[1,])
   n<-as.numeric(colSums(x))
   names(resp) <- names(n) <- dimnames(x)[[2]]
   success <- dimnames(x)[[1]][1]
   }
  else
   {
   
   success<-as.character(success)
   if(length(success)!=1)
     {stop("success must be a single character string")}

   if(all(dimnames(x)[[1]]!=success))
     {stop(paste("success=", success,"can not be found in", names(dimnames(x)), collapse="" ))}

   resp <- as.numeric(x[success,])
   n<-as.numeric(colSums(x))
   names(resp) <- names(n) <- dimnames(x)[[2]]
   }
 }
 else
  {
  if(is.null(success))
   {
   resp <- as.numeric(x[,1])
   n <- as.numeric(rowSums(x))
   names(resp) <- names(n) <- dimnames(x)[[1]]
   success <- dimnames(x)[[2]][1]
   }
  else
   {
   
   success<-as.character(success)
   if(length(success)!=1)
     {stop("success must be a single character string")}

   if(all(dimnames(x)[[2]]!=success))
     {stop(paste("success=", success,"can not be found in", names(dimnames(x)), collapse="" ))}

   resp <- as.numeric(x[,success])
   n <- as.numeric(rowSums(x))
   names(resp) <- names(n) <- dimnames(x)[[1]]
   } 

  }

out <- binomest.default(x=resp, n=n, names=names(resp), method=method, success=success)

return(out)

}

`Waldtest` <-
function(estp, varp, cmat, alternative="greater", dist="MVN")
{

 k <- ncol(cmat)
 m <- nrow(cmat) 

if(any(c(length(estp), length(varp))!=k))
 {stop("estp, varp and ncol(cmat) must be of equal length")}
 
dist<-match.arg(dist, choices=c("MVN", "N"))

# point estimate:
 estC <- cmat %*% estp

# Variance estimates for the contrasts using unpooled (groupwise) variance estimates:
 varC <- (cmat^2) %*% (varp)  

# Teststatistic:

 teststatv <- estC/sqrt(varC)
 
switch(dist,
"MVN"={

CorrMat<-corrMatgen(CM=cmat, varp=varp)

p.val.adj <- numeric(length=m) 

# adjusted p.values for the single hypotheses

for( i in 1:m)
 {

if(alternative=="two.sided")
 {teststati <- abs(teststatv[i])
 p.val.adj[i] <- 1 - pmvnorm(lower=rep(-teststati,m), upper=rep(teststati,m), mean=rep(0,m), corr=CorrMat)}

if(alternative=="greater")
 {teststati <- teststatv[i]
  p.val.adj[i] <- 1 - pmvnorm(lower=rep(-Inf,m), upper=rep(teststati,m), mean=rep(0,m), corr=CorrMat)}

if(alternative=="less")
 {teststati <- teststatv[i]
  p.val.adj[i] <- 1 - pmvnorm(lower=rep(teststati,m), upper=rep(Inf,m), mean=rep(0,m), corr=CorrMat)}
 
 }

pmaxtest<-min(p.val.adj)

},
"N"={
p.val.adj <- numeric(length=m) 

# adjusted p.values for the single hypotheses

for( i in 1:m)
 {

if(alternative=="two.sided")
 {teststati <- abs(teststatv[i])
 p.val.adj[i] <- min(1, 2*(1 - pnorm(q=teststati)))}

if(alternative=="greater")
 {teststati <- teststatv[i]
  p.val.adj[i] <- 1 - pnorm(q=teststati)}

if(alternative=="less")
 {teststati <- teststatv[i]
  p.val.adj[i] <- pnorm(q=teststati)}
 
 }

pmaxtest<-NA

})

return(
list(teststat=teststatv, pval=pmaxtest, p.val.adj=p.val.adj, alternative=alternative, dist=dist)
)
}

lnrest <-
function(x, f)
{
    f <- as.factor(f)
    nx <- tapply(x, f, length)
    lx <- log(x)
    mlx <- tapply(lx,f,mean)
    varlx <- tapply(lx,f,var)*((nx-1)/nx)
    lmx <- mlx + 0.5 * varlx
    varlmx <- varlx/nx + (varlx^2)/(2*nx)
return(list(estimate=lmx, varest=varlmx, n=nx))}

`censsample` <-
function(n, scale.m, shape.m, scale.t, shape.t=3, tmax )
{

if( any(c(length(n), length(scale.m), length(shape.m), length(scale.t), length(shape.t))!=1) )
 {stop("input single numbers")}

# T.m time of mortality
# T.t time of tumor onset

T.m <- rweibull(n=n, shape=shape.m, scale=scale.m)      ###### Zeit bis Tod
T.t <- rweibull(n=n, shape=shape.t, scale=scale.t)      ###### Zeit bis Tumor = Tod

# at time of death of an animal dies: is a tumor present or not:
# time at which animals die or are sacrificed min(T.t, T.m)
# status=TRUE if before alternative death or final sacrifice a tumor developed
# status is FALSE if time of tumor onset T.t is greater than tmax or T.m

mat<-cbind(T.t, T.m, tmax)
time <- apply(X=mat, MARGIN=1, FUN=function(x){min(x[2:3])})
status <- apply( X=mat, MARGIN=1, FUN=function(x){ x[1] < min(x[2:3]) } )
return(data.frame(time=time, status=status, T.t=T.t, T.m=T.m, tmax = tmax))

}

Y2mat <- function(Ymat){
  if(is.table(Ymat)){
    if(length(dim(Ymat))!=2){
      stop(paste("Ymat should be a two-dimensional contingency table but input has", length(dim(Ymat)), "dimensions"))}else{YMAT <- as.matrix(Ymat)}}else{
                   if(is.data.frame(Ymat)){
                     wcn <- unlist(lapply(as.list(Ymat), function(x){is.numeric(x)|is.integer(x)}))
                     if(length(wcn)>sum(wcn)){warning("Some columns in Ymat do not contain numbers and will be omitted.")}
                     if(sum(wcn)>=2){YMAT <- data.matrix(Ymat[,wcn])}else{stop("Ymat should contain at least 2 columns with numeric/integer entries")}
                   }else{
                     if(is.matrix(Ymat)){
                       if(length(dim(Ymat))!=2){stop(paste("Ymat should be a two-dimensional contingency table but input has",
                                                                 length(dim(Ymat)), "dimensions"))}else{ YMAT <- Ymat}
                     }else{stop("Ymat should be a 2-dimensional matrix but input can not be coerced to a 2 dimensional matrix")}
                     
                   }
                 }
 
  if(is.numeric(YMAT)|is.integer(YMAT)){return(as.matrix(YMAT))}else{stop("Ymat must have integer (or numeric) entries")} 
  
}



datcmcheck <- function(Ymat, cmcat=NULL, cmgroup=NULL, ...)
{
  
  Ymat <- Y2mat(Ymat)
  if(!(is.numeric(Ymat)|is.integer(Ymat))){stop("Ymat must have integer (or numeric entries)")}
  if(any(Ymat-round(Ymat) >=  10 * .Machine$double.eps)){warning("Ymat should have whole numbers as entries")}

    ncat <- ncol(Ymat)
    ngrp <- nrow(Ymat)
    
    catni <- colSums(Ymat)
    groupni <- rowSums(Ymat)    
    
  if(is.null(cmcat)){
    CMCAT <- contrMat(n=catni, type="Dunnett", ...) 
  }else{
  if(is.character(cmcat)){
    ctype <- match.arg(cmcat, choices=c("Dunnett", "Tukey", "Sequen"))
    CMCAT <- contrMat(n=catni, type=ctype, ...)
  }else{
  if(!is.matrix(cmcat)){stop("cmcat must be a matrix or a single character string")}else{
  if(!(is.numeric(cmcat)|is.integer(cmcat))){stop("cmcat must have integer (or numeric entries)")}else{
  CMCAT <- cmcat  
  }}}}

    if(is.null(cmgroup)){
      CMGROUP <- contrMat(n=groupni, type="Dunnett", ...) 
    }else{
      if(is.character(cmgroup)){
        ctypegroup <- match.arg(cmgroup, choices=c("Dunnett", "Tukey", "Sequen"))
        CMGROUP <- contrMat(n=groupni, type=ctypegroup, ...)
      }else{
        if(!is.matrix(cmgroup)){stop("cmgroup must be a matrix or a single character string")}else{
          if(!(is.numeric(cmgroup)|is.integer(cmgroup))){stop("cmgroup must have integer (or numeric entries)")}else{
            CMGROUP <- cmgroup
          }}}}
    

    if(ncol(CMCAT) != ncat){stop("Number of columns in cmcat must be the same as number of columns (categories) in Ymat")}
    if(ncol(CMGROUP) != ngrp){stop("Number of columns in cmgroup must be the same as number of rows (groups) in Ymat")}
    
    RScat <- rowSums(CMCAT)
    RScatpos <- apply(CMCAT, 1, function(x){sum(x[sign(x)==+1])})
    
    RSgrp <- rowSums(CMGROUP)
    RSgrppos <- apply(CMGROUP, 1, function(x){sum(x[sign(x)==+1])})
    
    if(any(RScat>=10 * .Machine$double.eps)){warning("Some rows of 'contrast' matrix cmcat do not sum to 0.")}
    if(any(RSgrp>=10 * .Machine$double.eps)){warning("Some rows of 'contrast' matrix cmgrp do not sum to 0.")}
 
    if(any((RScatpos-1)>=10 * .Machine$double.eps)){cat("Note: The positive coefficients in some rows of 'contrast' matrix cmcat do not sum to 0.")}
    if(any((RSgrppos-1)>=10 * .Machine$double.eps)){cat("Note: The positive coefficients in some rows of 'contrast' matrix cmgrp do not sum to 0.")}
  return(list(Ymat=Ymat, cmgroup=CMGROUP, cmcat=CMCAT))  
}
  
  


##################################
# helper functions

addmat <- function(Ymat, alpha=1, type="group"){
  ngroup <- nrow(Ymat); ncat <- ncol(Ymat) 
  switch(type,
         "group"={priormat <- matrix(rep(alpha/ncat, times=ngroup*ncat),ncol=ncat)},
         "cell"={priormat <- matrix(rep(alpha, times=ngroup*ncat),ncol=ncat)},
         "total"={priormat <- matrix(rep(alpha/(ncat*ngroup), times=ngroup*ncat),ncol=ncat)})
  return(priormat)
}


makediriprior <- function(Ymat, prior){
if(is.null(prior)){PRIOR <- addmat(Ymat=Ymat, alpha=1, type="cell")}else{
  
  if(!(is.numeric(prior)|is.integer(prior))){stop("prior must be numeric")}

  ncat <- ncol(Ymat)
  ngrp <- nrow(Ymat)
  
  if(!is.matrix(prior)){
     if(length(prior)==1){ PRIOR <- addmat(Ymat=Ymat, alpha=prior, type="cell")}else{
     if(length(prior)==ncat){ PRIOR <- matrix(rep(prior, times=nrow(Ymat)), ncol=ncat)}else{
     if(length(prior)==(ncat*ngrp)){PRIOR <- matrix(prior, ncol=ncat)}else{stop("Prior is not a single number nor a vector, nor a matrix")
     }}}
  }else{
    if(ncol(prior)==ncol(Ymat) & nrow(prior)==nrow(Ymat)){
    PRIOR <- prior
    }else{stop("prior does not match dimensions of Ymat, and can not be coerced to match it")}}}
  if(is.null(rownames(PRIOR))){rownames(PRIOR) <- rownames(Ymat)}
  if(is.null(colnames(PRIOR))){colnames(PRIOR) <- colnames(Ymat)}
  return(PRIOR)
}


# paramat: a matrix with B=many samples 
# of parameters (columns) from a posterior distrribution
# or a bootstrapping process with B bootstrap replications

# CM a contrast matrix; ncol(CM) must be equal ncol(paramat)
# CM is %*%-multiplied with each row of paramat


# METHOD: sample from the dirichlet-posterior and
# compute SCS acc. to Besag et al. (1995)

# Ymat: data matrix
# prior: prior matrix
# cmcat: def. log-odds difference
# cmgroup def. between-group comparisons
# BSIM: No. of samples from dirichlet posterior

# bycomp: how to order parameters: bycomp=TRUE, parameters primarily ordered by betweengroup-comparisons, sec. by odds
#         bycomp=TRUE, parameters primarily ordered by betweengroup-comparisons, sec. by odds
#         bycomp=FALSE, parameters primarily ordered by odds, sec. by between-group-comparisons

# returns SCS at logit-scale!

############################################################

# SCI-method

simpostdiri <- function(Ymat, prior=NULL, cmcat=NULL, cmgroup=NULL, BSIM=10000,
 alternative="two.sided", conf.level=0.95, bycomp=FALSE, bychr=" btw ", ...){
 

YCG <- datcmcheck(Ymat=Ymat, cmcat=cmcat, cmgroup=cmgroup, ...)

YMAT <- YCG$Ymat
CMCAT <- YCG$cmcat
CMGROUP <- YCG$cmgroup

PRIOR <- makediriprior(Ymat=YMAT, prior=prior)

if(!is.numeric(BSIM)||length(BSIM)!=1){stop("BSIM must be a single, large integer")}
alternative <- match.arg(alternative, choices=c("two.sided", "less", "greater"))
if(!is.numeric(conf.level)||length(conf.level)!=1||(conf.level>=1 |conf.level<=0)){stop("conf.level must be a single numeric, e.g. 0.95")}


# apply CM%*%vector to each row of a 2-dimensional array (matrix)
CM1apply <- function(paramat, CM){apply(paramat, MARGIN=1, function(x){CM%*%matrix(x, ncol=1)})}

# result is a matrix with nrow=nrow(paramat) and ncol=nrow(CM)

# apply CM%*%vector to the third dimension vector of row of a 3-dimensional array
CM12apply <- function(paraarr, CM){apply(paraarr, MARGIN=1:2, function(x){CM%*%matrix(x, ncol=1)})}

# no. of groups in data
ngr <- nrow(YMAT)

# no. of categories in data
ncat <- ncol(YMAT)

# no. of odds (w.rt. categories)
nodds <- nrow(CMCAT)

# no. of between-group-comp.
ncomp <- nrow(CMGROUP)

compnames <- rownames(CMGROUP)
oddsnames <- rownames(CMCAT)

# use data + PRIOR as the dirichlet posterior
postpar <- YMAT+PRIOR

# define different 3-dimensional arrays

# sample of the dirichlet posterior
postarr <- array(dim=c(BSIM, ncat, ngr))

# sample of log-odds (for each treatment group separately)
postlogodds <- array(dim=c(BSIM, nodds, ngr))

# sample of log-odds differences (for each treatment group separately)
postdifflogit <- array(dim=c(BSIM, nodds, ncomp))

# sample of differences (comp. between treatments) for each log-odds
postcomplogit <- array(dim=c(BSIM, nodds*ncomp, ngr))

# sample from the dirichlet posterior, separately 
# 3 dim. array: BSIM x K x I
# (i.e. independently) for each treatment group (i=1,...,I)
for(i in 1:ngr){postarr[,,i] <- rdirichlet(n=BSIM, alpha=postpar[i,])}
for(i in 1:ngr){postlogodds[,,i] <- t(CM1apply(paramat=log(postarr[,,i]), CM=CMCAT))}

# compute samples of the between-group differences of the logits
for(d in 1:nodds){
postdifflogit[,d,] <- t(CM1apply(paramat=postlogodds[,d,], CM=CMGROUP))
}

# coerce the MxD matrix in each of the B samples into a vector of length=M*D

LNAM <- oddstrt2logitnames(cmcat=CMCAT, cmgroup=CMGROUP, bycomp=bycomp, bychr=bychr)

if(bycomp){
postlogitmat <- t(apply(X=postdifflogit, MARGIN=1, function(x){as.numeric(x)}))
colnames(postlogitmat) <- LNAM$logitnames
}else{
postlogitmat <- t(apply(X=postdifflogit, MARGIN=1, function(x){as.numeric(t(x))}))
colnames(postlogitmat) <- LNAM$logitnames}

SCS <- SCSrank(x=postlogitmat, conf.level=conf.level, alternative=alternative)

OUT<-list(SCS=SCS, Ymat=YMAT, prior=PRIOR, postarr=postarr, postlogodds=postlogodds, 
postdifflogit=postdifflogit, postlogitmat=postlogitmat)

return(c(OUT, LNAM))
}




##############################################

### Simultaneous Wald-type add-x-intervals: ##

##############################################

vcovA <- function(Yvecg, cma){
  ncat <- length(Yvecg)
  nyg <- sum(Yvecg)
  pi <- Yvecg/nyg
  dipi <- diag(1/pi)
  v1c <- matrix(rep(1, ncat), nrow=ncat)
 SIGMAg <- ((cma %*% dipi %*% t(cma))-(cma %*% v1c %*% t(v1c) %*% t(cma)))/nyg
return(SIGMAg)
}


vcovBLg <- function(Yvecg, base=1){
  nbl <- length(Yvecg)-1
  vcm <- matrix(1/Yvecg[base], nrow=nbl, ncol=nbl)
  diag(vcm) <- 1/Yvecg[base] + 1/Yvecg[-base]
  return(vcm)
}


vcovBL <- function(Ymat, base=1){
  # list with groupwise vcov matrices
  matlist <- alply(.data=Ymat, .margins=1, .fun=vcovBLg, base=base) 
  # blockdiagonal matrix with groupwise mats in the diag
  bdmat <- do.call("adiag", args=matlist)
  return(bdmat)
}

oddstrt2logitnames <- function(cmcat, cmgroup, bycomp=TRUE, bychr=" btw ")
{
  
  if(is.null(rownames(cmcat))){rownames(cmcat) <- paste("odds", 1:nrow(cmcat), sep="")}
  if(is.null(rownames(cmgroup))){rownames(cmgroup) <- paste("comp", 1:nrow(cmgroup), sep="")}
  
  compnames <- rownames(cmgroup)
  oddsnames <- rownames(cmcat)
  nodds <- nrow(cmcat)
  ncomp <- nrow(cmgroup)
  
  if(bycomp){
    cnamout <- rep(compnames, each=nodds)
    onamout <- rep(oddsnames, times=ncomp)
    logitnames <- paste(onamout,cnamout,  sep=bychr)
  }else{
    cnamout <- rep(compnames, times=nodds)
    onamout <- rep(oddsnames, each=ncomp)
    logitnames <- paste( onamout,cnamout, sep=bychr)
    
  }
  return(list(logitnames=logitnames, compnames=cnamout, oddsnames=onamout, cmcat=cmcat, cmgroup=cmgroup))
}


waldaddx <- function(Ymat, addx=0, addtype="group",
 cmcat, cmgroup, alternative="two.sided",
 conf.level=0.95, base=1, bycomp=TRUE, bychr=" btw ", ...){
  
# flip the order of a vector (that resulted from as.vector(matrix))
# as if matrix would have been transposed
vecflip <- function(x, nrow, ncol){as.vector(t(matrix(x, nrow=nrow, ncol=ncol)))}

YCG <- datcmcheck(Ymat=Ymat, cmcat=cmcat, cmgroup=cmgroup, ...)

YMAT <- YCG$Ymat
CMCAT <- YCG$cmcat
CMGROUP <- YCG$cmgroup


if(!(is.numeric(addx)|is.integer(addx))){stop("addx must be a single number")}
if(length(addx)>1){warning("Only first entry of addx will be used"); addx<-addx[1]}
if(addx<0 | addx>1){warning("Quantity to be added to cells (addx) is outside [0,1]")}

alternative <- match.arg(alternative, choices=c("two.sided", "less", "greater"))
if(!is.numeric(conf.level)||length(conf.level)!=1||(conf.level>=1 |conf.level<=0)){stop("conf.level must be a single numeric, e.g. 0.95")}

  ADDMAT <- addmat(Ymat=YMAT, alpha=addx, type=addtype)
  Ymataddx <- YMAT + ADDMAT

# replace 0  
  if(any(Ymataddx <=10 * .Machine$double.eps)){
    Ymataddx[Ymataddx <=10 * .Machine$double.eps] <- 0.5
    evcovBL <- vcovBL(Ymataddx, base=base)
    replace0 <- TRUE
  }else{replace0 <- FALSE}
  
  ncat <- ncol(YMAT); cni <- rep(3,ncat)
  if(!is.null(colnames(YMAT))){names(cni)<-colnames(YMAT)}
  
  if(length(base)>1|!(is.numeric(base)|is.integer(base))){stop("base must be a single interger")}
  if(base>ncat|base<=0){stop(paste("There are ", ncat, " categories; base must be in [1,", ncat, ",]"))}
  
  cmBL <- contrMat(cni, type="Dunnett", base=base)
  
  #PsimatBL <- log(Ymataddx) %*% t(cmBL)
  PsimatBL <- cmBL %*% t(log(Ymataddx))
  PsivecBL <- as.vector(PsimatBL)
  evcovBL <- vcovBL(Ymataddx, base=base)
  
  cmcatBL <- CMCAT[,-base]
  
  cmLOR <- kronecker(CMGROUP, cmcatBL) 
  cmLOR

  npara <- nrow(cmLOR)
  
  #round(cbind(PsivecBL,  evcovBL),3)
  # estimate, corresp. covariance mat, and corresp. correlation matrix
  LORest <- cmLOR %*% matrix(PsivecBL, ncol=1)
  LORevcov <- cmLOR %*% evcovBL %*% t(cmLOR)
  LORecorrmat <- cov2cor(LORevcov)
  LORvarest <- diag(LORevcov)
  
  switch(alternative,
         "two.sided"={
           quanti <- qmvnorm(p = conf.level, sigma = LORecorrmat, tail = "both.tails")$quantile
           stderr <- sqrt(LORvarest)
           lCI <- LORest - quanti * stderr
           uCI <- LORest + quanti * stderr
         },
         "less"={
           quanti <- qmvnorm(p = conf.level, sigma = LORecorrmat,  tail = "lower.tail")$quantile
           stderr <- sqrt(LORvarest)
           lCI <- rep(-Inf, npara)
           uCI <- LORest + quanti * stderr
         },
         "greater"={
           quanti <- qmvnorm(p = conf.level, sigma = LORecorrmat,  tail = "upper.tail")$quantile
           stderr <- sqrt(LORvarest)
           lCI <- LORest + quanti * stderr
           uCI <- rep(Inf, npara)
         }
  )
  
  LNAM <- oddstrt2logitnames(cmcat=CMCAT, cmgroup=CMGROUP, bycomp=bycomp, bychr=bychr)
  SCS <- list()
  
  if(bycomp){
    SCS$conf.int <- cbind(lCI, uCI)
    colnames(SCS$conf.int) <- c("lower", "upper")
    rownames(SCS$conf.int) <- LNAM$logitnames
    SCS$estimate <- LORest
    names(SCS$estimate) <- LNAM$logitnames
    SCS$quanti <- quanti
    OUT<-list(SCS=SCS, Ymataddx=Ymataddx, addmat=ADDMAT, LORevcov=LORevcov, LORecorrmat=LORecorrmat, replace0=replace0)
  }
  if(!bycomp){
    flipid <- vecflip(x=1:length(LORest), nrow=nrow(cmcatBL), ncol=nrow(CMGROUP))
    SCS <- list()  
    SCS$conf.int <- cbind(lCI[flipid], uCI[flipid])
    colnames(SCS$conf.int) <- c("lower", "upper")
    rownames(SCS$conf.int) <- LNAM$logitnames
    SCS$estimate <- LORest[flipid]
    names(SCS$estimate) <- LNAM$logitnames
    SCS$quanti <- quanti

    OUT<-list(SCS=SCS, Ymataddx=Ymataddx, addmat=ADDMAT, LORevcov=LORevcov[flipid,flipid], LORecorrmat=LORecorrmat[flipid,flipid], replace0=replace0)
  }
  
return(c(OUT, LNAM))
  
}

  
multinomORci <- function(Ymat, cmcat=NULL, cmgroup=NULL, cimethod="DP", alternative="two.sided",
                                conf.level=0.95, bycomp=FALSE, bychr=" btw ", ...){
args<-list(...)
argnames <- names(args)

cimethod <- match.arg(cimethod, choices=c("DP", "Wald"))

if(is.null(args$base)){BASE <- 1}else{BASE <- args$base}

switch(cimethod,
       "DP"={
         if(any(!argnames %in% c("prior", "BSIM"))){
           warg <- which(!argnames %in% c("prior", "BSIM", "base"))
           warning(paste("Argument(s)", paste(argnames[warg], collapse=", "), " not available for method DP, and thus ignored"))}
         if(is.null(args$prior)){prior<-1}else{prior<-args$prior}
         if(is.null(args$BSIM)){BSIM<-10000}else{BSIM<-args$BSIM}
         RES <- simpostdiri(Ymat=Ymat, prior=prior, cmcat=cmcat, cmgroup=cmgroup,
                            BSIM=BSIM, alternative=alternative, conf.level=conf.level, bycomp=bycomp, bychr=bychr, base=BASE)
         METHODTXT <- paste(signif(conf.level*100, digits=3), "% SCI, ", BSIM, "samples from Dirichlet posterior" )
       },
       "Wald"={
         if(any(!argnames %in% c("addx", "addtype", "base"))){
           warg <- which(!argnames %in% c("addx", "addtype", "base"))
           warning(paste("Arguments", paste(argnames[warg], collapse=", "), " not available for method Wald, and thus ignored"))}
         if(is.null(args$addx)){addx<-0}else{addx<-args$addx}
         if(is.null(args$addtype)){addtype<-"cell"}else{addtype<-args$addtype}        
         RES <- waldaddx(Ymat=Ymat, cmcat=cmcat, cmgroup=cmgroup, alternative=alternative,
                           conf.level=conf.level, bycomp=bycomp, bychr=bychr, addx=addx, addtype=addtype,base=BASE)
         if(addx==0){ADDTXT <- ""}else{ADDTXT <- paste("(after adding", addx, "per", addtype, ")")}
         METHODTXT <- paste(signif(conf.level*100, digits=3), "% Wald-type SCI", ADDTXT)
       })
  
SCI <- cbind(data.frame(odds=RES$oddsname, comp=RES$compname, oddsratio=RES$logitnames#, estimate=RES$SCS$estimate
                        ), RES$SCS$conf.int)

OUT <-list(SCI=SCI, 
           details=c(RES, list(
          cimethod = cimethod,
          METHODTXT = METHODTXT
           )))
class(OUT) <- "multinomORci"

return(OUT)

}


as.data.frame.multinomORci <- function(x, row.names = NULL, optional = FALSE, exp=TRUE, ...){
dargs <- list(...)
dargs$row.names <- row.names
dargs$optional <- optional
DAT <- x$SCI
if(is.null(dargs$row.names)){row.names(DAT) <- NULL}
if(exp){
  DAT$lower <- exp(DAT$lower)
  DAT$upper <- exp(DAT$upper)
}
dargs$x <- DAT
do.call("as.data.frame", args=dargs)
}


print.multinomORci <- function(x, exp=TRUE, ...){
  dargs <- list(...)
  DAT <- as.data.frame(x, exp=exp)
  dargs$x <- DAT

  cat(x$details$METHODTXT, "\n")
  if(exp){cat("Intervals transformed to scale of odds ratios: \n")}else{cat("Intervals on logit-scale: \n")}
  do.call("print", args=dargs)
  
  if(x$details$cimethod == "DP") { cat("with Drichlet prior: \n")
    dargs$x <- x$details$prior
    do.call("print", args=dargs)}
  return(invisible(x))
}

'print.poly3est' <-
function(x, digits=4, ...)
{


if(x$method=="BP")
 {methvar<-"Bailer-Portier"}
if(x$method=="BW")
 {methvar<-"Bieler-Williams"}
if(x$method=="ADD1")
 {methvar<-"Add-1"}
if(x$method=="ADD2")
 {methvar<-"Add-2"}


#  Table of sample estimates

sample.estimate<-rbind(x$Y, x$n, x$nadj, x$estimate)
rownames(sample.estimate)<-c("x","n", "adjusted n", "adjusted estimate")

cat("Raw and poly-", x$k,"-adjusted sample estimates:","\n", sep="")
print(round(sample.estimate, digits=digits))
cat(" ","\n")

invisible(x)

}

\name{methyl}
\alias{methyl}
\docType{data}
\title{ NTP bioassay data: effect of methyleugenol on skin fibroma}
\description{
 NTP bioassay of methyleugenol: 200 male rats were randomly assigned to 4 treatment groups with balanced sample size 50.
 Individuals in treatment group 0, 1, 2, and 3 received doses of 0, 37, 75, and 150 mg methyleugenol per kg body weight, respectively.
 The response variable \kbd{tumour} is the presence of skin fibroma at time of death.
 The variable \kbd{death} gives individual time of death, with a final sacrifice of surviving animals at 730 days after begin of the assay.
}
\usage{data(methyl)}
\format{
  A data frame with 200 observations on the following 3 variables.
  \describe{
    \item{group}{a factor with levels \kbd{0}, \kbd{1}, \kbd{2}, \kbd{3}, specifying dose groups 0, 37, 75, and 150 mg/kg, respectively}
    \item{tumour}{a numeric vector, specifying whether a tumour was present at time of death}
    \item{death}{a numeric vector, specifying the time of death}
  }
}
\source{
  National toxicology program (2000).
}
\references{
  SD Peddada, GE Dinse, JK Haseman (2005): A survival-adjusted quantal response test for comparing tumour incidence rates. Applied Statistics 54, 51-61. 
}
\examples{
data(methyl)
# raw tumour proportions:
table(methyl[c("group", "tumour")])

# time of death:
boxplot(death~group, data=methyl, horizontal=TRUE)

}
\keyword{datasets}
\concept{poly-k}

\name{estSimpsonf}
\alias{estSimpsonf}

\title{ Estimate the Simpson index from several samples }
\description{
 Calculate estimates of the Simpson index after pooling over several samples, grouped by a factor variable.

}
\usage{
estSimpsonf(X, f)
}
\arguments{
  \item{X}{ a data.frame of dimension n times p with integer entries, where n is the number of samples and p is the number of species }
  \item{f}{ a factor variable of length n, grouping the observations in \kbd{X} }
}
\details{
 The function splits \kbd{X} according to the levels of the grouping variable \kbd{f}, builds the sum over each column and calculates the Shannon index ove the resulting counts.
 
}
\value{
 A list containing the items:
  \item{estimate }{ the groupwise point estimates of the Simpson index}
  \item{varest }{the groupwise variance estimates of the Simpson index}
  \item{table }{a matrix of counts,containing the summed observations for each level of \kbd{f} in its rows}
}
\references{ Rogers, JA and Hsu, JC (2001): Multiple Comparisons of Biodiversity. Biometrical Journal 43, 617-625. }


\seealso{ \link{estShannonf} }
\examples{

# Here, the estimates for the Hell Creek Dinosaur 
# example are compared to the estimates in 
# Tables 2 and 3 of Rogers and Hsu (2001).

data(HCD)
HCD

# Groupwise point estimates:

est<-estSimpsonf(X=HCD[,-1], f=HCD[,1])

est

# Table 2:


cmat<-rbind(
"lower-middle"=c(1,-1,0),
"lower-upper"=c(1, 0,-1),
"middle-upper"=c(0,1,-1))


# the point estimates:

# cmat %*% est$estimate
crossprod(t(cmat), est$estimate)


# the standard errors:
# sqrt(diag(cmat %*% diag(est$varest) %*% t(cmat)))

sqrt(diag(crossprod(t(cmat), crossprod(diag(est$varest), t(cmat)) ) ))


# Table 3:

cmat<-rbind(
"middle-lower"=c(-1,1,0),
"upper-lower"=c(-1,0,1))

# cmat %*% est$estimate
crossprod(t(cmat), est$estimate)


# sqrt(diag(cmat %*% diag(est$varest) %*% t(cmat)))

sqrt(diag(crossprod(t(cmat), crossprod(diag(est$varest), t(cmat)) ) ))

# Note, that the point estimates are exactly
# the same as in Rogers and Hsu (2001),
# but the variance estimates are not, whenever
# the Upper group is involved.


}

\keyword{ htest }
\concept{ biodiversity }

\name{estShannon}
\alias{estShannon}

\title{ Estimate the Shannon Index }
\description{
 Calculates estimates of the Shannon-Wiener from count data.
}
\usage{
estShannon(x, Nspec = NULL)
}

\arguments{
  \item{x}{ a integer(numeric) vector of species counts }
  \item{Nspec}{ a single integer value, fixing the number of species to a certain value }
}

\value{
A list, containing the elements:
  \item{estimate }{ a single numeric value, the estimate with bias correction according to Fritsch and Hsu (1999)}
  \item{estraw }{ a single numeric value, the raw estimate }
  \item{varest }{ a single numeric value, the variance estimate according to Fritsch and Hsu (1999) }

}
\references{ Fritsch, KS, and Hsu, JC (1999): Multiple Comparison
of Entropies with Application to Dinosaur Biodiversity. Biometrics 55, 1300-1305. }


\seealso{ \link{estSimpsonf} for estimating Shannon indices pooled over several samples, grouped by a factor  }


\keyword{ htest }
\concept{ biodiversity }

\name{summary.Simpsonci}
\alias{summary.Simpsonci}

\title{ Summary function for Simpsonci }
\description{
Produces a detailed print out of the results of function Simpsonci.
}
\usage{
\method{summary}{Simpsonci}(object, ...)
}

\arguments{
  \item{object}{ an object of class \code{"Simpsonci"} as obtained by calling \code{\link{Simpsonci}} }
  \item{\dots}{ further arguments to be passed to print and round: currently only \code{digits}}
}

\value{

A print out, comprising a table of the (possibly aggregated) data used for estimation,
the sample estimates for the Simpsons index, and its variance estimates,
the used contrast matrix, and the confidence intervals.

}

\examples{

data(HCD)

HCDcounts<-HCD[,-1]
HCDf<-HCD[,1]

SimpsonS<-Simpsonci(X=HCDcounts, f=HCDf, type = "Sequen",
 alternative = "greater", conf.level = 0.95, dist = "MVN")

summary(SimpsonS)

}


\keyword{ print }

\name{poly3estf}
\alias{poly3estf}
\title{ Only for internal use. }
\description{
 Poly-3- adjusted point and variance estimates for long term carcinogenicity data if data are given as a numeric time vector,
a logical status vector and a factor containing a grouping variable }
\usage{
poly3estf(time, status, tmax=NULL, f, method = "BP", k=NULL)
}

\arguments{
  \item{time}{  a numeric vector of times of death of the individuals}
  \item{status}{  a logical (or numeric, consisting of 0,1 only) vector giving the tumour status at time of death of each individual,
 where TRUE (1) = tumour present, FALSE (0) = no tumour present  }
  \item{tmax}{ a single numeric value, the time of sacrifice in the trial, or the last last time of death, defaults to the maximal value observed in \kbd{time} }
  \item{f}{  a factor of the same length as \kbd{time}, \kbd{status}, giving the levels of a grouping variable in a one-way layout }
  \item{method}{ a single charcter string, specifying the method for adjustment,
with options: "BP" (Bailer Portier: assuming poly-3-adjusted rates are binomial variables),
 "BW" (Bieler, Williams: delta method as in Bieler-Williams (1993))
"ADD1" (as Bailer Portier, including an add1-adjustment on the raw tumour rates)
"ADD2" (as Bailer Portier, including an add2-adjustment on the raw tumour rates following Agresti Caffo (2000) for binomials)
 }

\item{k}{a single numeric value, the exponent to calculate survival adjusted proportions according to Bailer and Portier (1988), defaults to 3}
}
\details{
 For internal use.
}
\value{
   A list containing:
  \item{Y }{a numeric vector, groupwise number of tumours}
  \item{n}{ a numeric vector, groupwise number of individuals}
  \item{estimate}{ a numeric vector, groupwise poly-3-adjusted rates according to Bailer, Portier (1988)}
  \item{weight}{a numeric vector of poly-3-adjusted weights }
  \item{estp}{a numeric vector, groupwise poly-3-adjusted rate (according to method)}
  \item{nadj}{adjusted n (sum of weights)}
  \item{varp}{a numeric vector, groupwise variance estimate (according to method)}
  \item{varcor}{a numeric vector, groupwise variance estimate, if necessary corrected such that estimates of 0 can not occur}
  \item{names}{a character vector, the levels of the grouping variable \kbd{f} }
  \item{k}{a single numeric value, as input}

}
\references{ See \link{poly3est}}

\note{ See \link{poly3est}}

\examples{

data(bronch)

poly3estf(status=bronch$Y, time=bronch$time, f=bronch$group, k=3)

poly3estf(status=bronch$Y, time=bronch$time, f=bronch$group, k=5)

}
\keyword{ htest }

\name{printfunctions}
\alias{print.binomORci}
\alias{print.binomRDci}
\alias{print.binomRDtest}
\alias{print.binomRRci}
\alias{print.poly3ci}
\alias{print.poly3est}
\alias{print.poly3test}
\alias{print.Shannonci}
\alias{print.Simpsonci}


\title{Print methods for the classes in this package}
\description{
Print methods for objects of the corresponding classes.
  }

\usage{
\method{print}{binomORci}(x, ...)
\method{print}{binomRDci}(x, digits = 4, ...)
\method{print}{binomRDtest}(x, digits = 4, ...)
\method{print}{binomRRci}(x, digits = 4, ...)
\method{print}{poly3ci}(x, digits = 4, ...)
\method{print}{poly3est}(x, digits = 4, ...)
\method{print}{poly3test}(x, digits = 4, ...)
\method{print}{Shannonci}(x, ...)
\method{print}{Simpsonci}(x, ...)
}

\arguments{
  \item{x}{ an object of the corresponding class }
  \item{digits}{the number of digits to be used for rounding }
  \item{\dots}{ ... }
}

\value{
A print out.
}


\keyword{ print }

\name{Waldci}
\alias{Waldci}
\title{ Simultaneous Wald confidence intervals }
\description{
  General function for simultaneous CIs in a one-way layout using multivariate normal distribution.

}
\usage{
Waldci(cmat, estp, varp, varcor, alternative = "two.sided", conf.level = 0.95, dist="MVN")
}

\arguments{
  \item{cmat}{ Contrast matrix of dimension MxI, with M = the number of contrasts, I= the number of samples }
  \item{estp}{ numeric vector of point estimates of length I, with I = the number of samples}
  \item{varp}{ numeric vector of variance estimates of length I, to be used for interval construction }
  \item{varcor}{ numeric vector of variance estimates of length I }
  \item{alternative}{ character string }
  \item{conf.level}{  single numeric vector }
  \item{dist}{a character string, \code{"MVN"} invokes multiplicity adjustment via the multivariate normal distribution,
\code{"N"} invokes use of quantiles of the univariate normal distribution}
}
\details{
Mainly for internal use.

}
\value{
A list containing:
  \item{conf.int }{a matrix with 2 columns: lower and upper confidence bounds, and M rows}
  \item{alternative }{character string, as input}
  \item{conf.level}{single numeric value, as input}
  \item{quantile}{the quantile used to construct the CIs}

}
\author{ Frank Schaarschmidt}

\seealso{For user level implementations see: 
\code{\link{binomRDci}}, 
\code{\link{binomORci}},
\code{\link{poly3ci}}
}

\keyword{ htest }

\name{PQln}
\alias{PQln}

\title{
Generalized pivotal quantity for log expected values of several lognormal samples.
}
\description{
Computes samples of the generalized pivotal quantities for the log of expected values of lognormal samples according to Krishnamoorthy and Mathew (2003). For internal use in \code{lndci}, \code{lnrci}.
}
\usage{
PQln(nx, mlx, varlx, B = 10000)
}

\arguments{
  \item{nx}{
 a numeric (integer) vector, the sample sizes for each of K groups
}
  \item{mlx}{
 a numeric vector, the sample estimates of the log mean for each of K groups
}
  \item{varlx}{
 a numeric vector, the (unbiased) sample estimates of the log variance for each of K groups
}
  \item{B}{
a single (integer) value, the number of samples to draw, defaults to 10000
}
}
\details{
Generates B samples from independent standard normal and chi^2 distributions for each of K independent groups in a one-way layout and computes the generalized pivotal quantities for the log expected value for each of the groups, according to Krishnamoorthy and Mathew (2003).
The input vectors \code{nx}, \code{mlx}, \code{varlx} must be vectors of length K.
}
\value{
A BxK matrix
}
\references{
Krishnamoorthy K, Mathew T (2003): Inferences on the means of lognormal distributions using generalized p-values and generalized confidence intervals. Journal of Statistical Planning and Inference 115,
103-121.
}
\author{
Frank Schaarschmidt
}

\keyword{ internal }

\name{mosaicdiv}
\alias{mosaicdiv}


\title{ Mosaicplot for the data in Shannonci and Simpsonci}
\description{
Create a mosaicplot from objects of class Shannonci or Simpsonci
}
\usage{
mosaicdiv(x, decreasing = NULL, ...)
}

\arguments{
  \item{x}{ an object of class \kbd{"Simpsonci"} or \kbd{"Shannonci"} as can be obtained from calling \link{Simpsonci} or \link{Shannonci} }
  \item{decreasing}{ a single logical value, indicating whether the species should be plotted in the current order (if \kbd{decreasing=NULL}),
 in decreasing order (if \kbd{decreasing=TRUE}), or in increasing order (if \kbd{decreasing=FALSE}) }
  \item{\dots}{ further arguments to be passed to \kbd{mosaicplot}, see \kbd{?mosaicplot} and \kbd{?par} for details }
}
\details{
 This function uses the counts in \kbd{[["sample.estimate"]][["table"]]} to produce a mosaicplot.
}
\value{
A plot.
}

\examples{

data(HCD)

HCDFam <- HCD[,-1]

SCI<-Simpsonci(X=HCDFam, f=HCD[,1])

mosaicdiv(SCI, decreasing=TRUE, col=rainbow(n=8))

}

\keyword{ hplot }
\concept{ diversity }

\name{Simpson}
\alias{Simpson}

\title{ Calculate the Simpson index }
\description{
 Calculate the Simpson index from a vector of proportions.
}
\usage{
Simpson(p)
}
\arguments{
  \item{p}{ a numeric vector of proportions }
}

\keyword{ internal }

\name{SCSrank}
\alias{SCSrank}
\title{Compute a rectangular simultaneous confidence set from a sample of a joint empirical distribution.}
\description{
Given a large sample of N values from an M dimensional joint empirical distribution, the rank based method of Besag et al. (1995) is used to compute a rectangular M-dimensional 'confidence' set
that includes N*conf.level values of the sample.
}
\usage{
SCSrank(x, conf.level = 0.95, alternative = "two.sided", ...)
}

\arguments{
  \item{x}{
an N x M matrix containg N sampled values of the M dimensional distribution of interest
}
  \item{conf.level}{
the simultaneous confidence level, a single numeric value between 0 and 1, defaults to 0.95 for simultaneous 95 percent sets
}
  \item{alternative}{
a single character string related to hypotheses testing, \code{"two.sided"} invokes two-sided confidence sets, \code{"less"} invokes sets with upper limits only and \code{"greater"} invokes sets with lower limits only, 
}
  \item{\dots}{ currently ignored}
}


\value{
an Mx2 (\code{alternative="two.sided"}) matrix containing the lower and upper confidence limist for the M dimensions,
in case of \code{alternative="less"}, \code{alternative="greater"} the lower and upper bounds are replaced by \code{-Inf} and \code{Inf}, respectively.
}
\references{
Besag J, Green P, Higdon D, Mengersen K (1995). Bayesian Computation and Stochastic Systems. Statistical Science 10, 3-66.
Mandel M, Betensky RA. Simultaneous confidence intervals based on the percentile bootstrap approach. Computational Statistics and Data Analysis 2008; 52(4): 2158-2165.
}
\author{
Frank Schaarschmidt
}

\examples{

x <- cbind(rnorm(1000,1,2), rnorm(1000,0,2), rnorm(1000,0,0.5), rnorm(1000,2,1))
dim(x)
cm <- rbind(c(-1,1,0,0), c(-1,0,1,0), c(-1, 0,0,1))
xd <- t(apply(x, 1, function(x){crossprod(t(cm), matrix(x))}))
pairs(xd)

SCSrank(xd, conf.level=0.9)

}

\keyword{ htest }

\name{summary.binomRDtest}
\alias{summary.binomRDtest}

\title{ Detailed print out for binomRDtest }
\description{
 Produces a more detailed print out of objects of class "binomRDtest",
 including summary statistics,
 the used contrast matrix and the p-values.
}
\usage{
\method{summary}{binomRDtest}(object, ...)
}
\arguments{
  \item{object}{ an object of class "binomRDtest" as created by function \link{binomRDtest}  }
  \item{\dots}{ further arguments to be passed to summary, currently only \kbd{digits} for rounding is supported }
}
\value{
 A print out.
}

\examples{


ntrials <- c(40,20,20,20)
xsuccesses <- c(1,2,2,4)
names(xsuccesses) <- LETTERS[1:4]
test<-binomRDtest(x=xsuccesses, n=ntrials, method="ADD1",
 type="Changepoint", alternative="greater")

test

summary(test)

}

\keyword{ print }

\name{summary.binomORci}
\alias{summary.binomORci}

\title{ Detailed print out for binomORci }
\description{
Produces a more detailed print out of objects of class "binomORci", including summary statistics,
 the used contrast matrix and the confidence intervals.
}
\usage{
\method{summary}{binomORci}(object, ...)
}
\arguments{
  \item{object}{ an object of class "binomORci" as created by function \link{binomORci} }
  \item{\dots}{ ... }
}

\value{
A print out.
}


\examples{

x<-c(1,3,6,7,5)
n<-c(30,30,30,30,30)
names<-LETTERS[1:5]

ORD<-binomORci(x=x, n=n, names=names,
 type="Dunnett", alternative="greater")
summary(ORD)

ORW<-binomORci(x=x, n=n, names=names,
 type="Williams", alternative="greater")
summary(ORW)


}
\keyword{ print }

\name{censsamplef}
\alias{censsamplef}
\title{ Random data for Poly-k  }
\description{
Random data for Poly-k for a one-way layout, with I groups.
}
\usage{
censsamplef(n, scale.m, shape.m, scale.t, shape.t = 3, tmax)
}

\arguments{
   \item{n}{ a numeric vector, the numbers of individuals of length I }
  \item{scale.m}{  a numeric vector, scale parameters of the Weibull distribution for mortality }
  \item{shape.m}{ a numeric vector, shape parameters of the Weibull distribution for mortality }
  \item{scale.t}{ a numeric vector, scale parameters of the Weibull distribution for tumour induction }
  \item{shape.t}{ a numeric vector, shape parameters of the Weibull distribution for tumour induction }
  \item{tmax}{ a single numeric value, maximum time in the trial }
}

\value{A data.frame with columns
   \item{time}{a numeric vector of length \kbd{n}, the time of death of an individual}
   \item{status}{a logical vector of length \kbd{n}, the tumour status at time of death (TRUE: tumour present, FALSE: no tumour present)}
   \item{T.t}{time of tumour induction (unobservable)}
   \item{T.m}{time of death}
   \item{tmax}{maximum time of death}
   \item{f}{a factor of containing an appropriate grouping variable}
}

\keyword{datagen}

\name{poly3est}
\alias{poly3est}

\title{ Only for internal use. }
\description{
 Poly-3- adjusted point and variance estimates for long term carcinogenicity data 
 }
\usage{
poly3est(time, status, tmax, method = "BP", k=NULL)
}
\arguments{
  \item{time}{  a numeric vector of times of death of the individuals}
  \item{status}{  a logical (or numeric, consisting of 0,1 only) vector giving the tumour status at time of death of each individual,
 where TRUE (1) = tumour present, FALSE (0) = no tumour present  }

  \item{tmax}{ a single numeric vector, the final time of the trial }
  \item{method}{ a single charcter string, specifying the method for adjustment,
with options: "BP" (Bailer Portier: assuming poly-3-adjusted rates are binomial variables),
 "BW" (Bieler, Williams: delta method as in Bieler-Williams (1993))
"ADD1" (as Bailer Portier, including an add1-adjustment on the raw tumour rates)
"ADD2" (as Bailer Portier, including an add2-adjustment on the raw tumour rates following Agresti Caffo (2000) for binomials)
 }
\item{k}{a single numeric value, the exponent to calculate survival adjusted proportions according to Bailer and Portier (1988), defaults to 3}
}
\details{
 Only for internal use of \link{poly3test} and \link{poly3ci}.
}
\value{
 A list containing:
  \item{Y }{number of tumours}
  \item{n}{number of individuals}
  \item{estimate}{poly-3-adjusted rates according to Bailer, Portier (1988)}
  \item{weight}{a vector of poly-3-adjusted weights, of length n }
  \item{estp}{poly-3-adjusted rate (according to method)}
  \item{nadj}{adjusted n (sum of weights)}
  \item{varp}{ variance estimate (according to method)}
  \item{varcor}{ variance estimate, if necessary corrected such that estimates of 0 can not occur}
}
\references{
 Bailer, J.A. and Portier, C.J. (1988):
 Effects of treatment-induced mortality and tumor-induced mortality on tests for carcinogenicity in small samples.
 Biometrics 44, 417-431.}

\note{ Please note, that appropriateness of the point estimates seriously depends on whether the assumptions in Bailer and Portier are met or not. }

\keyword{misc}

\name{poly3table}
\alias{poly3table}
\title{ Summarize long term carcinogenicity data }
\description{
   Function to summarize data of long term carcinogenicity trials in a text format.
 Data are assumed to consist of (1) a dichotomous variable, 
 defining whether the tumour of interest was present in an individual animal at time of death,
 and (2) a numeric variable containing the time of death of an individual animal,
 and (3) a grouping factor.
}
\usage{
poly3table(time, status, f, tumour = NULL, symbol = "*")
}
\arguments{
  \item{time}{ a numeric vector, containing the time of death of an individual }
  \item{status}{ a logical (or dichotomous categorical) vector }
  \item{f}{ a factor, specifying treatment groups}
  \item{tumour}{ the value which \kbd{status} obtains if a tumour is present in an individual at time of death  }
  \item{symbol}{ symbol to indicate presence of tumour in the text representation }
}

\value{ A named list, containing a character string for each group
}

\author{ Frank Schaarschmidt }
\examples{
data(methyl)
methyl
poly3table(time=methyl$death, status=methyl$tumour,
 f=methyl$group, tumour = 1, symbol = "*")

}

\keyword{ print }
\concept{poly-k}

\name{estSimpson}
\alias{estSimpson}

\title{ Simpson index }
\description{
 Calculates the estimate of the Simpson index from a vector of counts.
}
\usage{
estSimpson(x)
}

\arguments{
  \item{x}{a integer (numeric) vector, giving the counts of p species in a community }
}


\keyword{ htest }
\concept{ biodiversity }

\name{plot.sci}
\alias{plot.sci}
\alias{plot.binomRDci}
\alias{plot.binomORci}
\alias{plot.binomRRci}
\alias{plot.Shannonci}
\alias{plot.Simpsonci}

\alias{plot.poly3ci}

\title{ Plot confidence intervals }
\description{
Function for convenient plotting of confidence intervals.
}
\usage{
\method{plot}{sci}(x, ...)
\method{plot}{binomRDci}(x, ...)
\method{plot}{binomORci}(x, ...)
\method{plot}{binomRRci}(x, ...)
\method{plot}{poly3ci}(x, ...)
\method{plot}{Shannonci}(x, ...)
\method{plot}{Simpsonci}(x, ...)

}

\arguments{
  \item{x}{ an object of class \kbd{"binomRDci"}, \kbd{"binomORci"}, \kbd{"binomRRci"}, \kbd{"poly3ci"}, \kbd{"sci"} }
  \item{\dots}{ further arguments as described in \link{plotCI} }
} 
\details{
Extracts some values and calls \link{plotCI}.

}
\value{
A plot.
}

\keyword{ hplot }

\name{liarozole}
\alias{liarozole}
\docType{data}
\title{ Marked improvement of psoriasis after application of liarozole}
\description{
  In a placebo controlled clinical trial, patients with psoriasis were randomly assigned
 to a placebo group and three dose groups (50 mg, 75 mg, and 150 mg). Variable of primary
 interest was the proportion of patients with marked improvement of psoriasis.
 This data.frame mimics how raw data could have been represented in a larger data frame.
}
\usage{data(liarozole)}
\format{
  A data frame with 137 observations on the following 2 variables.
  \describe{
    \item{Improved}{a factor with levels \kbd{n}, \kbd{y}, for "no" and "yes"}
    \item{Treatment}{a factor with levels \kbd{Dose150}, \kbd{Dose50}, \kbd{Dose75}, \kbd{Placebo}}
  }
}
\details{
   For illustrative purpose only. Number of successes recalculated from proportions
 presented in the publication, while the number of patients in group \kbd{Dose50} was not exactly clear.
}
\source{
  Berth-Jones J, Todd G, Hutchinson PE, Thestrup-Pedersen K, Vanhoutte FP:
  Treatment of psoriasis with oral liarozole: a dose-ranging study.
  British Journal of Dermatology 2000; 143: 1170-1176.
}

\examples{
data(liarozole)
head(liarozole)
# create a contingency table:

table(liarozole)


# the order of the groups is alpha-numeric,
# and "y" for success is of higher order than
# to change the order:

liarozole$Treatment<-factor(liarozole$Treatment,
 levels=c("Placebo", "Dose50", "Dose75", "Dose150"))

liarozole$Improved<-factor(liarozole$Improved,
 levels=c("y", "n"))


tab<-table(liarozole)
tab

# Approximate simultaneous confidence intervals
# for the differences  pDose-pPlacebo:

LCI<-binomRDci(tab, type="Dunnett", 
alternative="greater", method="ADD1")

LCI

plot(LCI, main="Proportion of patients
 with marked improvement")

# Perform a test on increasing trend 
# vs. the placebo group:

Ltest<-binomRDtest(tab, type="Williams", 
alternative="greater", method="ADD1")

summary(Ltest)

}
\keyword{datasets}
\concept{binomial}
\concept{proportion}

\name{Waldtest}
\alias{Waldtest}

\title{ Simultaneous Wald tests}
\description{
  General function for adjusted p-values for an UIT in a one-way layout using multivariate normal distribution.
}
\usage{
Waldtest(estp, varp, cmat, alternative = "greater", dist="MVN")
}
\arguments{ 
  \item{estp}{ numeric vector of point estimates of length I, with I = the number of samples}
  \item{varp}{ numeric vector of variance estimates of length I, to be used for interval construction }
  \item{cmat}{ Contrast matrix of dimension MxI, with M = the number of contrasts, I= the number of samples }
  \item{alternative}{ character string }
 \item{dist}{a character string, where \code{"MVN"} invokes the computation of p-values using the multivariate normal distribution,
 and \code{"N"} invokes use p-value computation using the univariate normal distribution}
}

\value{
A list containing:
\item{teststat}{a numeric vector of teststatistics of length M}
\item{pval}{a single numeric p-value, the p-value of the maximum test (minimum p-value)} 
\item{p.val.adj}{ a vector of length M, the adjusted p-values of the single contrasts}
\item{alternative}{a single character vector, as the input}
\item{dist}{a character string specifying which distribution}
}


\author{ Frank Schaarschmidt }

\seealso{
For user level implementations see:

\code{\link{binomRDtest}},
\code{\link{poly3test}}

}

\keyword{ htest }

\name{HCD}
\alias{HCD}
\docType{data}
\title{ Hell Creek Dinosaur Data}
\description{
 Counts of dinosaur families found in three stratigraphic levels of the Cretaceous period in the Hell Creek formation in North Dakota.
 The eight families are the Ceratopsidae (Ce), Hadrosauridae (Ha), Hypsilophodontidae (Hy), Pachycephalosauridae (Pa), Tyrannosauridae (Ty), Ornithomimidae
(Or), Sauronithoididae (Sa) and Dromiaeosauridae (Dr).

}
\usage{data(HCD)}
\format{
  A data frame with 3 observations on the following 9 variables.
  \describe{
    \item{Level}{a factor with levels \kbd{Lower}, \kbd{Middle}, \kbd{Upper}, specifiyng the stratigraphic levels}
    \item{Cr}{a numeric vector, counts of the Ceratopsidae }
    \item{Ha}{a numeric vector, counts of the Hadrosauridae}
    \item{Hy}{a numeric vector, counts of the Hypsilophodontidae}
    \item{Pa}{a numeric vector, counts of the Pachycephalosauridae}
    \item{Ty}{a numeric vector, counts of the Tyrannosauridae}
    \item{Or}{a numeric vector, counts of the Ornithomimidae}
    \item{Sa}{a numeric vector, counts of the Sauronithoididae}
    \item{Dr}{a numeric vector, counts of the Dromiaeosauridae}
  }
}

\source{
 Table 1 in: Rogers, JA and Hsu, JC (2001): Multiple Comparisons of Biodiversity. Biometrical Journal 43, 617-625.
}
\references{
  Sheehan, P.M., et al. (1991): Sudden extinction of the Dinosaurs: Latest Cretaceous, Upper Great Plains, U.S.A. Science 254, 835-839.
}
\examples{

data(HCD)
str(HCD)
HCD

mat<-as.matrix(HCD[,-c(1)])

rownames(mat)<-HCD$Level

mosaicplot(mat, las=1)

estSimpsonf(X=HCD[,-c(1)], f=HCD$Level)

estShannonf(X=HCD[,-c(1)], f=HCD$Level)

}
\keyword{datasets}

\name{censsample}
\alias{censsample}

\title{ Random data for Poly-k }
\description{
Random numbers from two independent Weibull distributions for Mortality and tumour induction.
}
\usage{
censsample(n, scale.m, shape.m,
 scale.t, shape.t = 3, tmax)
}
\arguments{
  \item{n}{ a single numeric value, the number of individuals }
  \item{scale.m}{  a single numeric value, scale parameter of the Weibull distribution for mortality }
  \item{shape.m}{ a single numeric value, shape parameter of the Weibull distribution for mortality }
  \item{scale.t}{ a single numeric value, scale parameter of the Weibull distribution for tumour induction }
  \item{shape.t}{ a single numeric value, shape parameter of the Weibull distribution for tumour induction }
  \item{tmax}{ a single numeric value, maximum time in the trial }
}

\value{ A data.frame with columns
   \item{time }{a numeric vector of length \kbd{n}, the time of death of an individual}
   \item{status }{a logical vector of length \kbd{n}, the tumour status at time of death (TRUE: tumour present, FALSE: no tumour present)}
   \item{T.t}{time of tumour induction (unobservable)}
   \item{T.m}{time of death}
   \item{tmax}{maximum time of death}
}

\keyword{datagen}

\name{Shannon}
\alias{Shannon}

\title{ Shannon index }
\description{
 Calculates the Shannon-Wiener index.
}
\usage{
Shannon(p)
}

\arguments{
  \item{p}{A numeric vector of proportions. }
}



\keyword{ internal }

\name{summary.poly3est}
\alias{summary.poly3est}

\title{ Detailed print out for poly3est }
\description{
Summary statistics for long-term carcinogenicity data, including poly-3-estimates.
For internal use.
}
\usage{
\method{summary}{poly3est}(object, ...)
}

\arguments{
  \item{object}{ An object of class "poly3est", as can be obtained by \code{\link{poly3est}} }
  \item{\dots}{ further argument for the print out, as e.g. \code{digits} for rounding }
}
\details{For internal use.}

\value{
A print out.
}

\author{ Frank Schaarschmidt }

\examples{

data(methyl)
head(methyl)

estk3<-poly3estf(time=methyl$death, status=methyl$tumour, f=methyl$group)
summary(estk3)

estk5<-poly3estf(time=methyl$death, status=methyl$tumour, f=methyl$group, k=5)
summary(estk5)

}

\keyword{ print }

\name{lndest}
\alias{lndest}
\alias{lnrest}

\title{
Point and varioance estimates for expected and log(expected values) of lognormal samples
}
\description{
Compute the ML estimates of the expected values and the log of the expected values given several samples of lognormal data, separated by a factor variable. For internal use in \code{lndci} and \code{lnrci}.
}
\usage{
lndest(x, f)
lnrest(x, f)
}

\arguments{
  \item{x}{
a numeric vector 
}
  \item{f}{
a factor variable (with at least two levels)
}
}

\value{
A list with elements
  \item{estimate }{ a numeric vector, the point estimates for each factor level}
  \item{varest }{ a numeric vector, the variance estimates (evaluated at the point estimates) for each factor level}
  \item{n }{ a numeric vector, the sample size of each factor level}
}
\references{
Chen, Y-H, Zhou, X-H (2006). Interval estimates for the ratio and the difference of two log-normal means. Statistics in Medicine 25, 4099-4113.
}

\keyword{ internal }

\name{plotCI}

\alias{plotCI}
\alias{plotCI.default}
\alias{plotCI.sci}
\alias{plotCI.sci.ratio}
\alias{plotCI.confint.glht}

\title{ Plot confidence intervals }
\description{
 A function for convenient plotting of confidence intervals.
}
\usage{
\method{plotCI}{default}(x, ...)
\method{plotCI}{sci}(x, ...)
\method{plotCI}{sci.ratio}(x, ...)
\method{plotCI}{confint.glht}(x, ...)
}

\arguments{
  \item{x}{ An object of class "sci", "sci.ratio" or "conf.int.glht" or a list with elements \kbd{estimate}, containing a numeric vector,\kbd{conf.int}, containing a matrix with two columns, giving the lower and upper bounds, and a string \kbd{alternative}, one of "two.sided", "less", "greater"  }
  \item{\dots}{ additional arguments to be passed to \link{plotCII} and \link{plot}, see \link{plotCII} for details}
}
\details{
 Plots the estimates, upper and lower limits using \kbd{points} and \kbd{segments}. The names of \kbd{estimate} are passed as labels of the confidence intervals.
 If infinite bounds occur, the plot region is limited by the most extreme non infinite bound or estimate.
}
\value{
  A plot.
}


\seealso{ Internally, the function \link{plotCII} is used.}
\examples{

x=c(8,9,9,18,39,44)
n=c(2000,2000,2000,2000,2000,2000)

x<-binomORci(x=x, n=n, names=c("0","120","240","480","600","720"))

plotCI(x, lines=1)


}

\keyword{ hplot }

\name{bronch}
\alias{bronch}
\docType{data}
\title{Rodent bronchial carcinoma data}
\description{
 In a long term rodent carcinogenicity study on female B6C3F1 mice, 
 the effect of vinylcyclohexene diepoxide on the incidence of murine
 alveolar/bronchiolar tumors was assessed. The mice were exposed to
 0 mg/ml, (group 0), 25 mg/ml (group 1), 50 mg/ml (group 2),
 and 100 (group 3) mg/ml, with 50 animals per group.
}
\usage{data(bronch)}
\format{
  A data frame with 200 observations on the following 3 variables.
  \describe{
    \item{group}{a factor with levels 0, 1, 2, 3, labelling the control and the three dose groups}
    \item{Y}{a logical vector, indicating whether a tumour was present at time of death (if TRUE), or not (if FALSE) }
    \item{time}{a numeric vector, the time of death, counted in days? from begin of the study}
  }
}
\details{
  Not yet checked for consistency with the source!
}
\source{
Piegorsch WW and Bailer AJ (1997): Statistics for environmental biology and toxicology.
Chapman and Hall, London.
Table 6.5, page 238.

}
\references{
Portier cJ and Bailer AJ (1989): Testing for increased carcinogenicity using survival-adjusted quantal response tests.
Fundamental and Applied Toxicology 12, 731.
}
\examples{
data(bronch)
# raw tumour counts:

table(bronch[c("group","Y")])

# groupwise times of death:

boxplot(time ~ group, data=bronch, horizontal=TRUE)

# Using poly3estf, we can produce the
# summary statistics as presented in 
# Table 6.6, page 239, of Piegorsch and Bailer (1997):

poly3estf(status=bronch$Y, time=bronch$time, f=bronch$group)


}
\keyword{datasets}

\name{summary.binomRDci}
\alias{summary.binomRDci}
\title{ Detailed print out for binomRDci }
\description{
 Produces a more detailed print out of objects of class "binomRDci", including summary statistics,
 the used contrast matrix and the confidence intervals.
}
\usage{
\method{summary}{binomRDci}(object, ...)
}
\arguments{
  \item{object}{ an object of class "binomRDci" as created by function \link{binomRDci} }
  \item{\dots}{ further arguments to be passed to summary, currently only \kbd{digits} for rounding is supported }
}

\value{
 A print out.
}

\examples{



data(liarozole)

head(liarozole)

LiWi<-binomRDci(Improved ~ Treatment, data=liarozole,
 type="Williams")

LiWi

summary(LiWi)



}

\keyword{ print }

\name{summary.binomRRci}
\alias{summary.binomRRci}
\title{ Detailed print out for binomRRci }
\description{
 Produces a more detailed print out of objects of class "binomRRci", including summary statistics,
 the used contrast matrix and the confidence intervals.
}
\usage{
\method{summary}{binomRRci}(object, ...)
}
\arguments{
  \item{object}{ an object of class "binomRRci" as created by function \code{\link{binomRRci}} }
  \item{\dots}{ further arguments to be passed to summary, currently only \code{digits} for rounding is supported }
}

\value{
 A print out.
}

\examples{



data(liarozole)

head(liarozole)

LiDu<-binomRRci(Improved ~ Treatment, data=liarozole,
 type="Dunnett", alternative="greater")

LiDu

summary(LiDu)



}

\keyword{ print }

\name{print.multinomORci}
\alias{print.multinomORci}
\alias{as.data.frame.multinomORci}
\title{
Print out put of multinomORci
}
\description{
For output of function multinomORci: print ot confidence intervals or coerce out put to a data.frame.
}
\usage{
\method{print}{multinomORci}(x, exp = TRUE, ...)
\method{as.data.frame}{multinomORci}(x, row.names = NULL, optional = FALSE, exp = TRUE, ...)
}

\arguments{
  \item{x}{an object of class \code{multinomORci}}
  \item{exp}{logical; if \code{exp=TRUE} interval limits are exp-transformed to yield limits on the sclae of odds-ratios; if \code{exp=FALSE} interval limits are printed/Returned on the logit scale}
  \item{row.names}{see \code{as.data.frame}}
  \item{optional}{see \code{as.data.frame}}
  \item{\dots}{arguments to be passed to \code{print}, or \code{as.data.frame}}
}

\examples{

# fakle data: 3 categories (A,B,C) in 4 tretament groups c(co,d1,d2,d3)
dm <- rbind(
"co" = c(15,5,5),
"d1" = c(10,10,5),
"d2" = c(5,10,10),
"d3" = c(5,5, 15))
colnames(dm)<- c("A","B","C")

dm

#  define and name odds between categories
cmodds <- rbind(
"B/A"=c(-1,1,0),
"C/A"=c(-1,0,1))

#  define and name comparsions between groups
cmtrt <- rbind(
"d1/co"=c(-1,1,0,0),
"d2/co"=c(-1,0,1,0),
"d3/co"=c(-1,0,1,0))

TEST <- multinomORci(Ymat=dm, cmcat=cmodds, cmgroup=cmtrt, cimethod="DP", BSIM=1000, prior=1)
TEST
print(TEST, exp=FALSE)
as.data.frame(TEST)

}

\keyword{ misc }

\name{corrMatgen}
\alias{corrMatgen}

\title{ A function to calculate the correlation matrix }
\description{
Correlation matrix for teststatistics and confidence intervals assuming multivariate standard normal distribution
}
\usage{
corrMatgen(CM, varp)
}
\arguments{
  \item{CM}{ a matrix of contrast coefficients, dimension MxI, where M=number of contrasts, and I=number of groups in a oneway layout }
  \item{varp}{ a numeric vector of groupwise variance estimates (length = I) }
}

\value{
A matrix of dimension MxM. 

}

\references{
For correlation of contrasts of binomial proportion, see:
 Bretz F, Hothorn L.:
 Detecting dose-response using contrasts: asymptotic power and sample size determination for binomial data.
 Statistics in Medicine 2002; 21: 3325-3335. }

\keyword{ INTERNAL }

\name{estShannonf}
\alias{estShannonf}

\title{ Estimate the Shannon-Wiener index }
\description{
  Calculate estimates of the Shannon-Wiener index after pooling over several samples, grouped by a factor variable.
}
\usage{
estShannonf(X, f)
}

\arguments{
  \item{X}{ a data.frame of dimension n times p with integer entries, where n is the number of samples and p is the number of species}
  \item{f}{ a factor variable of length n, grouping the observations in \kbd{X} }
}
\details{
 The function splits \kbd{X} according to the levels of the grouping variable \kbd{f}, builds the sum over each column and calculates the Shannon index ove the resulting counts.
 
}
\value{
A list, containing the elements:
 \item{estimae }{ a named numeric vector, the groupwise Shannon indices with bias correction according to Fritsch and Hsu (1999)}
 \item{estraw }{ a named numeric vector, the groupwise Shannon indices, without bias correction}
 \item{varest }{ a named numeric vector, the groupwise variance estimates of the Shannon indices}
 \item{table }{a matrix, giving the summarized counts of the groups in the rows}
}
\references{ Fritsch, KS, and Hsu, JC (1999): Multiple Comparison
of Entropies with Application to Dinosaur Biodiversity. Biometrics 55, 1300-1305. }

\examples{

data(HCD)
HCD

# Groupwise point estimates:

est<-estShannonf(X=HCD[,-1], f=HCD[,1])

est



}

\keyword{ htest }
\concept{biodiversity }

\name{scalechoice}
\alias{scalechoice}

\title{ Only for internal use.}
\description{
How to define scale parameters of the Weibull distribution in the poly-k-setting  }
\usage{
scalechoice(shape = 3, pm = 0.2, t = 1)
}

\arguments{
  \item{shape}{ shape parameter (usually shape=3) }
  \item{pm}{ a numeric value (0<p<1), probability that an individual dies before time t }
  \item{t}{ a numeric value, time t }
}
\value{
  a single numeric value, giving the scale parameter needed to fulfill pm(t) for the current definition of the Weibull disstribution.}


\note{
Scale parameter is defined differently than that in Peddada (2005)
 }
\seealso{ \kbd{?pweibull} }

\keyword{ internal }
